<?php

defined('BASEPATH') OR exit('No direct script access allowed');

require APPPATH.'controllers/General.php'; 

class Account extends General {

	private $editPermission;

	function __construct() {
		parent::__construct();	

		if(General::isLoggedIN() === FALSE OR General::validateToken() === FALSE){
            redirect(base_url());    
        }
        
        $this->load->model('user_model');
	}

	public function settings()
	{
		$this->data['page_title'] = 'Account Settings';
		$this->data['page_sub_title'] = '';
		$this->data['css'] = $this->assets_resources->getAssets('account_settings_asset')['index']['css'];
		$this->data['js'] = $this->assets_resources->getAssets('account_settings_asset')['index']['js'];
		$this->data['jsextend'] = $this->assets_resources->getAssets('account_settings_asset')['index']['jsextend'];
		$this->data['file'] = $this->assets_resources->getAssets('account_settings_asset')['index']['file'];
		$this->load->view("page", $this->data);
	}

	public function save_profile()
	{
		General::addJSONResponseHeader();

		try {
			$this->form_validation->set_rules("firstname","First Name","trim|xss_clean|required");
			$this->form_validation->set_rules("lastname","Last Name","trim|xss_clean|required");
			$this->form_validation->set_rules("phone","Contact No.","trim|xss_clean");
			$this->form_validation->set_rules("email","Email Address","trim|xss_clean|valid_email|required");
			$this->form_validation->set_rules("username","Username","trim|xss_clean|required");
			$this->form_validation->set_error_delimiters('', '');

			if($this->form_validation->run())
			{	
				$result = $this->user_model->save_profile();
				if($result) 
				{
					$this->session->set_userdata('username', $this->input->post('username'));

					// Record activity log
					$log = '<strong>'.General::getFullName().'</strong> updated profile.';
					General::log('ACCOUNT SETTINGS', 'UPDATE', $log);

					$data_arr = array('success' => TRUE, 'message' => 'You have successfully updated your profile');

				} else {
					$data_arr = array('success' => FALSE, 'message' => 'Their\'s an error in updating your profile.');
				}
			}
			else
			{
				$data_arr = array('success' => FALSE, 'message' => validation_errors());
			}
		} catch(Exception $e) {
			$data_arr = array("success" => FALSE, "message" => $e->getMessage());
		}

		die(json_encode($data_arr));
	}

	public function upload_picture()
	{
		General::addJSONResponseHeader();

		try {
			if (empty($_FILES['profile_picture']['name']))
			{
				$data_arr = array('success' => FALSE, 'message' => 'Please upload an image.');
			}
			else
			{
				$filename = 'user-'.strtotime(date("His"));

				$config = array(
					'allowed_types'		=>		'jpg|jpeg|png',
					'upload_path'		=>		realpath('public/images/users'),
					'max_size'			=>		2000,
					'file_name'			=>		$filename,
					'overwrite'			=>		TRUE
				);

				$this->load->library('upload', $config);

				if ($this->upload->do_upload('profile_picture'))
		        {
		            $upload_data = $this->upload->data();
		            General::resize_img('public/images/users/' . $upload_data['file_name'],'40','40');
		            $result = $this->user_model->upload_picture($upload_data);
		            if($result) {
		            	$data_arr = array('success' => TRUE, 'message' => 'Your profile picture has been uploaded.');
		            } else {
		            	$data_arr = array('success' => FALSE, 'message' => 'Their\'s an error in uploading your profile picture.');
		            }

		        } else {
		        	$data_arr = array('success' => FALSE, 'message' =>  $this->upload->display_errors());
		        }
			}
		} catch(Exception $e) {
			$data_arr = array("success" => FALSE, "message" => $e->getMessage());
		}

		die(json_encode($data_arr));
	}

	public function change_password()
	{
		General::addJSONResponseHeader();

		try {
			$this->form_validation->set_rules("current_password","Current Password","trim|required");
			$this->form_validation->set_rules("new_password","New Password","trim|required|min_length[6]");
			$this->form_validation->set_rules("retype_password","Re-type Password","trim|required|matches[new_password]");
			$this->form_validation->set_error_delimiters('', '');

			if($this->form_validation->run())
			{	
				$rsUserDetails = $this->user_model->getUserDetails($this->session->userdata('username'));
				$result = $this->password->check_password($this->input->post('current_password'), $rsUserDetails->password);
				if($result) 
				{
					$hashPassword = $this->password->hash($this->input->post('new_password'));
					$rsResult = $this->user_model->change_password($hashPassword);
					if($rsResult){

						$this->data['newpassword'] = $this->input->post('new_password');
						$body_email = $this->load->view('email_template/password_changed', $this->data, true);
						$this->email->set_mailtype("html");
						$this->email->from('aspendevelopmentinc@gmail.com', 'Admin Development Corporation');
						$this->email->to($rsUserDetails->email);
						$this->email->subject(General::SHORT_SYS_TITLE . ' - Password Changed');
						$this->email->message($body_email);
						$result = $this->email->send();

						$data_arr = array('success' => TRUE, 'message' => 'Your have successfully changed your password.');
					} else {
						$data_arr = array('success' => FALSE, 'message' => 'Their\'s an error in changing your password.');
					}
					
				} else {
					$data_arr = array('success' => FALSE, 'message' => 'Current password did not match.');
				}	
			}
			else
			{
				$data_arr = array('success' => FALSE, 'message' => validation_errors());
			}
		} catch(Exception $e) {
			$data_arr = array("success" => FALSE, "message" => $e->getMessage());
		}

		die(json_encode($data_arr));
	}

}
