<?php

defined('BASEPATH') OR exit('No direct script access allowed');


require_once  APPPATH . '/third_party/PhpWord/Autoloader.php';
use PhpOffice\PhpWord\Autoloader;
use PhpOffice\PhpWord\Settings;
Autoloader::register();
Settings::loadConfig();

require APPPATH.'controllers/General.php'; 

class Appointments extends General {

	private $editPermission;

	function __construct() {
		parent::__construct();	

		if(General::isLoggedIN() === FALSE OR General::validateToken() === FALSE){
            redirect(base_url());    
        }

        General::ModAccess(__CLASS__);
        self::getPermissionAccess();
        
        $this->load->model('appointment_model');
        $this->load->model('patients_model');
        $this->load->model('user_model');
        $this->session->set_userdata('parent_id', 'appointments');
	}

	private function getPermissionAccess()
	{
		$this->data['addPermission'] = General::validatePermission('add', FALSE);
        $this->data['deletePermission'] = General::validatePermission('delete', FALSE);
        $this->editPermission = General::validatePermission('edit', FALSE);
        $this->data['editPermission'] = $this->editPermission;
	}

	public function index()
	{
		$this->data['page_title'] = 'Appointments';
		$this->data['page_sub_title'] = 'Manage Patients';
		$this->data['css'] = $this->assets_resources->getAssets('appointments_asset')['index']['css'];
		$this->data['js'] = $this->assets_resources->getAssets('appointments_asset')['index']['js'];
		$this->data['jsextend'] = $this->assets_resources->getAssets('appointments_asset')['index']['jsextend'];
		$this->data['file'] = $this->assets_resources->getAssets('appointments_asset')['index']['file'];
		$this->data['dt'] = array('datatable' => FALSE, 'path' => strtolower(__CLASS__) . '/manage', 'sort' => array(1, 'asc'));
		$this->data['patientList'] = $this->patients_model->getAll('', '', '', TRUE);
		$this->data['doctorList'] = $this->user_model->getUserByGroup(5);

		$this->load->view("page", $this->data);
	}

	public function manage()
	{
		if (isset($_REQUEST['actionType']) && $_REQUEST['actionType'] == 'group_action') 
		{
			General::processAction('patients', 'id');
			self::datatable();
		} else if(isset($_REQUEST['action']) && $_REQUEST['action'] == 'filter') {
			self::datatable($search = TRUE);
		} else {
			self::datatable();
		}
	}

	private function datatable($search = FALSE)
	{
		$result = $this->appointment_model->getAll('', '', $search, TRUE);
		$iTotalRecords = count($result);
		$iDisplayLength = intval($_REQUEST['length']);
		$iDisplayLength = $iDisplayLength < 0 ? $iTotalRecords : $iDisplayLength; 
		$iDisplayStart = intval($_REQUEST['start']);
		$sEcho = intval($_REQUEST['draw']);

		$records = array();

		$end = $iDisplayStart + $iDisplayLength;
		$end = $end > $iTotalRecords ? $iTotalRecords : $end;

		$rsData = $this->appointment_model->getAll($end, $iDisplayStart, $search, TRUE);

		if($rsData) {
			foreach ($rsData as $rsData) 
			{
				$hideEdit = ($this->editPermission) ? "hide" : "";

				$status = ($rsData->nStatus) ? "success" : "danger";
				$statusName = ($rsData->nStatus) ? "Active" : "InActive";
				$records["data"][] = array(
					'<label class="mt-checkbox mt-checkbox-single mt-checkbox-outline"><input name="id[]" type="checkbox" class="checkboxes" value="'.$rsData->id.'"/><span></span></label>',
					$rsData->cmid,
					$rsData->firstname.' '.$rsData->lastname,
					$rsData->gender,
					$rsData->home_address.' '.$rsData->city.' '.$rsData->province.' '.$rsData->postal_code,
					'<span class="label label-sm label-'.$status.'">'.$statusName.'</span>',
					'<a href="'.base_url(__CLASS__ . "/edit/" . $rsData->id).'" class="btn btn-xs btn-outline grey-salsa"><i class="fa fa-edit"></i> Edit</a>'
				);
			}
		} else {
			$records["data"][] = array('','','','','','','');
		}

		$records["draw"] = $sEcho;
		$records["recordsTotal"] = $iTotalRecords;
		$records["recordsFiltered"] = $iTotalRecords;

		echo json_encode($records);
	}

	public function edit($id = '')
	{
		General::validatePermission(__FUNCTION__);

		if(isset($_POST['patient_id']) AND $_POST['patient_id'] != '') 
		{
			self::editsave();
		} 
		else 
		{
			$id = (int) $id;
			$this->data['page_title'] = 'Edit Appointment';
			$this->data['page_sub_title'] = 'Appointments';
			$this->data['css'] = $this->assets_resources->getAssets('appointments_asset')['add']['css'];
			$this->data['js'] = $this->assets_resources->getAssets('appointments_asset')['add']['js'];
			$this->data['jsextend'] = $this->assets_resources->getAssets('appointments_asset')['add']['jsextend'];
			$this->data['file'] = DIRECTORY_SEPARATOR . 'appointment' . DIRECTORY_SEPARATOR . __FUNCTION__ . '.php';
			$this->data['rsDetails'] = $this->appointment_model->getDetails($id);
			$this->data['patientList'] = $this->patients_model->getAll('', '', '', TRUE);
			$this->load->view("page", $this->data);
		}
	}

	public function editsave()
	{
		General::addJSONResponseHeader();

		try {
			$this->form_validation->set_rules("patient_id","Patient","trim|xss_clean|required");
			$this->form_validation->set_rules("start_date","Start Date","trim|xss_clean|required");
			$this->form_validation->set_rules("end_date","End Date","trim|xss_clean|required");
			$this->form_validation->set_rules("assgined_doctor","Assigned Doctor","trim|xss_clean|required");
			$this->form_validation->set_rules("appointment_reason","Appointment Remarks","trim|xss_clean");
			$this->form_validation->set_error_delimiters('', '');

			if($this->form_validation->run())
			{	
				$result = $this->appointment_model->save('update');
				if($result) 
				{
					// Record activity log
					$log = 'New Appointment: with id#:<strong>'.$result.'</strong> has been updated by ' .General::getFullName();
					General::log('APPOINTMENTS', 'UPDATE', $log);

					$data_arr = array('success' => TRUE, 'message' => 'Appointment has been updated.');

				} else {
					$data_arr = array('success' => FALSE, 'message' => 'Their\'s an error in saving appointment.');
				}
			}
			else
			{
				$data_arr = array('success' => FALSE, 'message' => validation_errors());
			}
		} catch(Exception $e) {
			$data_arr = array("success" => FALSE, "message" => $e->getMessage());
		}

		die(json_encode($data_arr));
	}

	public function add()
	{
	 	General::validatePermission(__FUNCTION__);

		if(isset($_POST['patient_id']) AND $_POST['patient_id'] != '') 
		{
			self::save();
		} 
		else 
		{
			$this->data['page_title'] = 'Add New Appointment';
			$this->data['page_sub_title'] = 'Appointments';
			$this->data['css'] = $this->assets_resources->getAssets('appointments_asset')['add']['css'];
			$this->data['js'] = $this->assets_resources->getAssets('appointments_asset')['add']['js'];
			$this->data['jsextend'] = $this->assets_resources->getAssets('appointments_asset')['add']['jsextend'];
			$this->data['file'] = $this->assets_resources->getAssets('appointments_asset')['add']['file'];
			$this->data['patientList'] = $this->patients_model->getAll('', '', '', TRUE);
			$this->load->view("page", $this->data);
		}
	}

	public function validate_designation()
	{
		$data = array(
				'edit'		=>	(isset($_POST['designation_id']) AND $_POST['designation_id'] != '') ? array('designation_id',$this->input->post('designation_id')) : FALSE,
				'table'		=>	'designation',
				'fields'	=>	array('designation_name' => $this->input->post('designation_name')),
		);
		if(General::validateFields($data) === TRUE) {
			$this->form_validation->set_message( __FUNCTION__ ,'Designation Already Exists.');
			return FALSE;
		} else {
			return TRUE;
		}
	}

	public function save()
	{
		General::addJSONResponseHeader();

		try {
			$this->form_validation->set_rules("patient_id","Patient","trim|xss_clean|required");
			$this->form_validation->set_rules("start_date","Start Date","trim|xss_clean|required");
			$this->form_validation->set_rules("end_date","End Date","trim|xss_clean|required");
			$this->form_validation->set_rules("assgined_doctor","Assigned Doctor","trim|xss_clean|required");
			$this->form_validation->set_rules("appointment_reason","Appointment Remarks","trim|xss_clean");
			$this->form_validation->set_error_delimiters('', '');

			if($this->form_validation->run())
			{	
				$result = $this->appointment_model->save('insert');
				if($result) 
				{
					// Record activity log
					$log = 'New Appointment: with id#:<strong>'.$result.'</strong> has been added by ' .General::getFullName();
					General::log('APPOINTMENTS', 'INSERT', $log);

					$data_arr = array('success' => TRUE, 'message' => 'Appointment has been saved', 'formaction' => 'reset');

				} else {
					$data_arr = array('success' => FALSE, 'message' => 'Their\'s an error in saving patient.');
				}
			}
			else
			{
				$data_arr = array('success' => FALSE, 'message' => validation_errors());
			}
		} catch(Exception $e) {
			$data_arr = array("success" => FALSE, "message" => $e->getMessage());
		}

		die(json_encode($data_arr));
	}

	public function addNewQuick()
    {
    	General::validatePermission('add');

		if(isset($_POST['designation_name']) AND $_POST['designation_name'] != '') 
		{
			self::save();
		} 
		else 
		{
			$this->data['form_title'] = $this->lang->line('manage_designation_addnew');
        	$this->load->view('quick_forms/add_designation',$this->data);
		}
    }

    public function loadListCalendar()
	{
		$activitiesList = $this->appointment_model->loadDataCalendar();

		echo json_encode($activitiesList);
	}

	public function getAllPhysicians()
	{
		$phycisians_list = $this->appointment_model->getAllPhysicians();

		echo json_encode($phycisians_list);
	}

	public function validateDate()
	{
		General::addJSONResponseHeader();

		$datetime = $_POST['d'].' '.$_POST['t'];
		$hasEndDate = $this->db->query("SELECT `end_date` as num FROM `appointment` WHERE `end_date` = '".$datetime."' AND `nStatus` = 1");

		// If has end date then add +5 mins.
		if($hasEndDate->num_rows() > 0) {
			$sTime = date("h:i", strtotime("+5 minutes", strtotime($_POST['t'])));
			$startTime = date("h:i", strtotime("+5 minutes", strtotime($_POST['t'])));
		} else {
			$sTime = $_POST['t'];
			$startTime = $_POST['t'];
		}

		$eTime = date("h:i", strtotime("+60 minutes", strtotime($sTime)));
		do {
			$sTime = date("H:i", strtotime("+5 minutes", strtotime($sTime)));
			$searchDate = $this->db->query("SELECT * FROM `appointment` WHERE `start_date` = '".$_POST['d'].' '.$sTime."' AND `nStatus` = 1");
			if($searchDate->num_rows() == 1) {
				$eTime = date("H:i", strtotime("-5 minutes", strtotime($sTime)));
				break;
			}
		} while($sTime != "23:55");


		$data = array('sTime'=>$startTime,'eTime'=>$eTime,'sDate'=>$_POST['d']);
		die(json_encode($data));
	}
	
	public function add_appointment()
	{
		General::addJSONResponseHeader();

		try {
			$this->form_validation->set_rules("patient_id","Patient","trim|xss_clean|required");
			$this->form_validation->set_rules("start_time","Start Time","trim|xss_clean|required");
			$this->form_validation->set_rules("assgined_doctor","Assigned Doctor","trim|xss_clean|required");
			$this->form_validation->set_rules("appointment_reason","Appointment Remarks","trim|xss_clean");
			$this->form_validation->set_rules("duration","Duration","trim|xss_clean|required");
			$this->form_validation->set_rules("appointment_type","Appointment Type","trim|xss_clean|required");
			$this->form_validation->set_error_delimiters('', '');

			$duration = intval($this->input->post('duration')) - 1;
			$end = strtotime("+$duration minutes", strtotime($this->input->post('start_time')));
			$end = date('H:i', $end);

			if($this->form_validation->run())
			{	
				$startTime = $this->input->post('date').' '.$this->input->post('start_time');
				$endTime = $this->input->post('date').' '.$end;

				$hasConflict = $this->db->query("SELECT * FROM `appointment` 
					WHERE (`start_date` > '".$startTime."' AND `start_date` < '".$endTime."' OR `end_date` > '".$startTime."' 
								AND `end_date` < '".$endTime."') 
					AND nStatus = 1 AND assgined_doctor = ".$this->input->post('assgined_doctor'));
				if($hasConflict->num_rows() > 0) {
					$data_arr = array('success' => FALSE, 'message' => 'Their\'s an conflict in your selected date. The selected date is already booked by other appintments, please pick other time.');
				} else {
					$result = $this->appointment_model->save('insert', $end);
					if($result) 
					{
						// Record activity log
						$log = 'New Appointment: with id#:<strong>'.$result.'</strong> has been added by ' .General::getFullName();
						General::log('APPOINTMENTS', 'INSERT', $log);

						$data_arr = array('success' => TRUE, 'message' => 'Appointment has been saved', 'formaction' => 'add_appointment');

					} else {
						$data_arr = array('success' => FALSE, 'message' => 'Their\'s an error in saving patient.');
					}
				}

				
			}
			else
			{
				$data_arr = array('success' => FALSE, 'message' => validation_errors());
			}
		} catch(Exception $e) {
			$data_arr = array("success" => FALSE, "message" => $e->getMessage());
		}

		die(json_encode($data_arr));
	}

	public function edit_appointment()
	{
		General::addJSONResponseHeader();

		try {
			$this->form_validation->set_rules("patient_id","Patient","trim|xss_clean|required");
			$this->form_validation->set_rules("start_time","Start Time","trim|xss_clean|required");
			$this->form_validation->set_rules("end_time","End Time","trim|xss_clean|required");
			$this->form_validation->set_rules("assgined_doctor","Assigned Doctor","trim|xss_clean|required");
			$this->form_validation->set_rules("appointment_reason","Appointment Remarks","trim|xss_clean");
			$this->form_validation->set_rules("duration","Duration","trim|xss_clean|required");
			$this->form_validation->set_rules("appointment_type","Appointment Type","trim|xss_clean|required");
			$this->form_validation->set_error_delimiters('', '');

			$duration = intval($this->input->post('duration')) - 1;
			$end = strtotime("+$duration minutes", strtotime($this->input->post('start_time')));
			$end = date('H:i', $end);

			if($this->form_validation->run())
			{	
				$startTime = $this->input->post('date').' '.$this->input->post('start_time');
				$endTime = $this->input->post('date').' '.$end;

				$sql = "SELECT * FROM `appointment` 
					WHERE (`start_date` > '".$startTime."' AND `start_date` < '".$endTime."' OR `end_date` > '".$startTime."' 
								AND `end_date` < '".$endTime."') 
					AND nStatus = 1 AND assgined_doctor = ".$this->input->post('assgined_doctor')." AND `id` != " . $this->input->post('appointment_id');
				$hasConflict = $this->db->query($sql);
				if($hasConflict->num_rows() > 0) {
					$data_arr = array('success' => FALSE, 'message' => 'Their\'s an conflict in your selected date. The selected date is already booked by other appintments, please pick other time.');
				} else {
					$result = $this->appointment_model->save('update', $end);
					if($result) 
					{
						// Record activity log
						$log = 'New Appointment: with id#:<strong>'.$result.'</strong> has been updated by ' .General::getFullName();
						General::log('APPOINTMENTS', 'UPDATE', $log);

						$data_arr = array('success' => TRUE, 'message' => 'Appointment has been updated', 'formaction' => 'edit_appointment');

					} else {
						$data_arr = array('success' => FALSE, 'message' => 'Their\'s an error in updating patient.');
					}
				}

				
			}
			else
			{
				$data_arr = array('success' => FALSE, 'message' => validation_errors());
			}
		} catch(Exception $e) {
			$data_arr = array("success" => FALSE, "message" => $e->getMessage());
		}

		die(json_encode($data_arr));
	}

	public function getAppDetails($id)
	{
		General::addJSONResponseHeader();
		$rsDetails = $this->appointment_model->getDetails($id);
		$rsDetails->date = date('Y-m-d', strtotime($rsDetails->start_date));
		$rsDetails->start_time = date('H:i', strtotime($rsDetails->start_date));
		$rsDetails->end_time = date('H:i', strtotime($rsDetails->end_date));
		die(json_encode($rsDetails));
	}

	public function deleteApp($id)
	{
		$rsDetails = $this->appointment_model->deleteApp($id);
		$data_arr = array('success' => TRUE, 'message' => 'Appointment has been deleted');
	}

	public function create_biiling_sheet($date, $doctor_id)
	{
		$year = explode('_', $date)[0];
		$month = explode('_', $date)[1];
		$day = explode('_', $date)[2];

		$doctorInfo = $this->user_model->getUserDetails('',$doctor_id);


		$this->load->library('excel');
		$this->excel->setActiveSheetIndex(0);

		$this->excel->getActiveSheet()->setTitle('Billing Sheet');
		$this->excel->getProperties()->setCreator(General::getCompanyInfo()->company_name)
									 ->setLastModifiedBy(General::getFullName())
									 ->setSubject("Billing Sheet Excel")
									 ->setTitle('Billing Sheet')
									 ->setDescription("Billing Sheet")
									 ->setCategory("Reports Generation");

		//set cell A1 content with some text
		$this->excel->getActiveSheet()->setCellValue('A1', date("Y-m-d").' - DR. '.$doctorInfo->firstname.' '.$doctorInfo->lastname);
		//change the font size
		$this->excel->getActiveSheet()->getStyle('A1')->getFont()->setSize(16);
		//merge cell A1 until D1
		$this->excel->getActiveSheet()->mergeCells('A1:I1');
		//set aligment to center for that merged cell (A1 to D1)

		$this->excel->getActiveSheet()->setCellValue('A2', 'Billing Sheet');
		$this->excel->getActiveSheet()->getStyle('A2')->getFont()->setSize(12);
		$this->excel->getActiveSheet()->mergeCells('A2:I2');


		// SET TABLE HEADER TO CENTER
		$this->excel->getActiveSheet()->getStyle('A4:H4')->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);

		// AUTO SIZE TABLE HEADER
		foreach(range('A4','H4') as $columnID) {
		    $this->excel->getActiveSheet()->getColumnDimension($columnID)
		        ->setAutoSize(true);
		}

		// SET BACKGROUND COLOR TABLE HEADER
		$this->excel->getActiveSheet()->getStyle('A4:H4')->getFill()->applyFromArray(array(
		        'type' => PHPExcel_Style_Fill::FILL_SOLID,
		        'startcolor' => array(
		             'rgb' => 'CCCCCC'
		        )
		    ));

		$this->excel->setActiveSheetIndex(0)
		                            ->setCellValue('A4', 'Appointment Start')
		                            ->setCellValue('B4', 'Patient Name')
		                            ->setCellValue('C4', 'DOB')
		                            ->setCellValue('D4', 'Year')
		                            ->setCellValue('E4', 'Age')
		                            ->setCellValue('F4', 'Condition')
		                            ->setCellValue('G4', 'Billing Code')
		                            ->setCellValue('H4', 'Diagnostic Code');

		// $data = $this->reports_model->clientAllExcel();
		$query = $this->db->query("SELECT B.`firstname`, B.`lastname`, B.`dob`,A.`start_date` 
				FROM appointment A
				LEFT OUTER JOIN patients B ON B.`id` = A.`patient_id` 
				WHERE A.`assgined_doctor` = ".$doctor_id." AND A.`nStatus` = 1 AND DATE_FORMAT(A.`start_date`, '%Y-%m-%d') = '".$year.'-'.$month.'-'.$day."' ORDER BY start_date ASC");
		$data = $query->result();
		$i = 5;
		foreach($data as $row){
			$age = date("Y") - date("Y", strtotime($row->dob));
			$age = General::computeAge($row->dob);
		    $this->excel->setActiveSheetIndex(0)
		            ->setCellValue('A'.$i, date('H:i A',strtotime($row->start_date)) )
		            ->setCellValue('B'.$i, $row->firstname.' '.$row->lastname)
		            ->setCellValue('C'.$i, $row->dob)
		            ->setCellValue('D'.$i, date("Y", strtotime($row->dob)))
		            ->setCellValue('E'.$i, $age)
		            ->setCellValue('F'.$i, '')
		            ->setCellValue('G'.$i, '')
		            ->setCellValue('H'.$i, '');
		    $i++;
		}

		$filename = date("Ymd")."_BillingSheet_DR".$doctorInfo->firstname.".xlsx";
		header('Content-Type: application/vnd.openxmlformats-officedocument.spreadsheetml.sheet'); //mime type
		header('Content-Disposition: attachment;filename="'.$filename.'"'); //tell browser what's the file name
		header('Cache-Control: max-age=0'); //no cache
		            
		//save it to Excel5 format (excel 2003 .XLS file), change this to 'Excel2007' (and adjust the filename extension, also the header mime type)
		//if you want to save it as .XLSX Excel 2007 format
		$objWriter = PHPExcel_IOFactory::createWriter($this->excel, 'Excel2007');  
		//force user to download the Excel file without writing it to server's HD
		$objWriter->save('php://output');
	}

	public function create_prescription_sheet($date, $doctor_id)
	{
		$year = explode('_', $date)[0];
		$month = explode('_', $date)[1];
		$day = explode('_', $date)[2];

		$doctorInfo = $this->user_model->getUserDetails('',$doctor_id);

		$this->load->library('excel');
		$this->excel->setActiveSheetIndex(0);

		$this->excel->getActiveSheet()->setTitle('Precription Details Sheet');
		$this->excel->getProperties()->setCreator(General::getCompanyInfo()->company_name)
									 ->setLastModifiedBy(General::getFullName())
									 ->setSubject("Precription Details Sheet")
									 ->setTitle('Precription Details Sheet')
									 ->setDescription("Precription Details Sheet")
									 ->setCategory("Reports Generation");

		//set cell A1 content with some text
		$this->excel->getActiveSheet()->setCellValue('A1', date("Y-m-d").' - DR. '.$doctorInfo->firstname.' '.$doctorInfo->lastname);
		//change the font size
		$this->excel->getActiveSheet()->getStyle('A1')->getFont()->setSize(16);
		//merge cell A1 until D1
		$this->excel->getActiveSheet()->mergeCells('A1:I1');
		//set aligment to center for that merged cell (A1 to D1)

		$this->excel->getActiveSheet()->setCellValue('A2', 'Prescription Details Sheet');
		$this->excel->getActiveSheet()->getStyle('A2')->getFont()->setSize(12);
		$this->excel->getActiveSheet()->mergeCells('A2:I2');


		// SET TABLE HEADER TO CENTER
		$this->excel->getActiveSheet()->getStyle('A4:G4')->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);

		// AUTO SIZE TABLE HEADER
		foreach(range('A4','G4') as $columnID) {
		    $this->excel->getActiveSheet()->getColumnDimension($columnID)
		        ->setAutoSize(true);
		}

		// SET BACKGROUND COLOR TABLE HEADER
		$this->excel->getActiveSheet()->getStyle('A4:G4')->getFill()->applyFromArray(array(
		        'type' => PHPExcel_Style_Fill::FILL_SOLID,
		        'startcolor' => array(
		             'rgb' => 'CCCCCC'
		        )
		    ));

		$this->excel->setActiveSheetIndex(0)
		                            ->setCellValue('A4', 'Appointment Start')
		                            ->setCellValue('B4', 'CMID')
		                            ->setCellValue('C4', 'Patient Name')
		                            ->setCellValue('D4', 'Daily Quantity')
		                            ->setCellValue('E4', 'Months')
		                            ->setCellValue('F4', 'THC Restriction')
		                            ->setCellValue('G4', 'Licensed Producer(s)');

		// $data = $this->reports_model->clientAllExcel();
		$query = $this->db->query("SELECT B.`cmid`, B.`firstname`, B.`lastname`, B.`dob`,A.`start_date` 
				FROM appointment A
				LEFT OUTER JOIN patients B ON B.`id` = A.`patient_id` 
				WHERE A.`assgined_doctor` = ".$doctor_id." AND A.`nStatus` = 1 AND DATE_FORMAT(A.`start_date`, '%Y-%m-%d') = '".$year.'-'.$month.'-'.$day."' ORDER BY start_date ASC");
		$data = $query->result();
		$i = 5;
		foreach($data as $row){
			$age = General::computeAge($row->dob);
		    $this->excel->setActiveSheetIndex(0)
		            ->setCellValue('A'.$i, date('H:i A',strtotime($row->start_date)) )
		            ->setCellValue('B'.$i, $row->cmid)
		            ->setCellValue('C'.$i, $row->firstname.' '.$row->lastname)
		            ->setCellValue('D'.$i, '')
		            ->setCellValue('E'.$i, '')
		            ->setCellValue('F'.$i, '')
		            ->setCellValue('G'.$i, '');
		    $i++;
		}

		$filename = date("Ymd")."_PrecriptionDetails_DR".$doctorInfo->firstname.".xlsx";
		header('Content-Type: application/vnd.openxmlformats-officedocument.spreadsheetml.sheet'); //mime type
		header('Content-Disposition: attachment;filename="'.$filename.'"'); //tell browser what's the file name
		header('Cache-Control: max-age=0'); //no cache
		            
		//save it to Excel5 format (excel 2003 .XLS file), change this to 'Excel2007' (and adjust the filename extension, also the header mime type)
		//if you want to save it as .XLSX Excel 2007 format
		$objWriter = PHPExcel_IOFactory::createWriter($this->excel, 'Excel2007');  
		//force user to download the Excel file without writing it to server's HD
		$objWriter->save('php://output');
	}

	public function create_appointment_sheet($date, $doctor_id)
	{
		$year = explode('_', $date)[0];
		$month = explode('_', $date)[1];
		$day = explode('_', $date)[2];

		$doctorInfo = $this->user_model->getUserDetails('',$doctor_id);

		$this->load->library('excel');
		$this->excel->setActiveSheetIndex(0);

		$this->excel->getActiveSheet()->setTitle('Appointment Details Sheet');
		$this->excel->getProperties()->setCreator(General::getCompanyInfo()->company_name)
									 ->setLastModifiedBy(General::getFullName())
									 ->setSubject("Appointment Details Sheet")
									 ->setTitle('Appointment Details Sheet')
									 ->setDescription("Appointment Details Sheet")
									 ->setCategory("Reports Generation");

		//set cell A1 content with some text
		$this->excel->getActiveSheet()->setCellValue('A1', date("Y-m-d").' - DR. '.$doctorInfo->firstname.' '.$doctorInfo->lastname);
		//change the font size
		$this->excel->getActiveSheet()->getStyle('A1')->getFont()->setSize(16);
		//merge cell A1 until D1
		$this->excel->getActiveSheet()->mergeCells('A1:I1');
		//set aligment to center for that merged cell (A1 to D1)

		$this->excel->getActiveSheet()->setCellValue('A2', 'Appointment Details Sheet');
		$this->excel->getActiveSheet()->getStyle('A2')->getFont()->setSize(12);
		$this->excel->getActiveSheet()->mergeCells('A2:I2');


		// SET TABLE HEADER TO CENTER
		$this->excel->getActiveSheet()->getStyle('A4:D4')->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);

		// AUTO SIZE TABLE HEADER
		foreach(range('A4','D4') as $columnID) {
		    $this->excel->getActiveSheet()->getColumnDimension($columnID)
		        ->setAutoSize(true);
		}

		// SET BACKGROUND COLOR TABLE HEADER
		$this->excel->getActiveSheet()->getStyle('A4:D4')->getFill()->applyFromArray(array(
		        'type' => PHPExcel_Style_Fill::FILL_SOLID,
		        'startcolor' => array(
		             'rgb' => 'CCCCCC'
		        )
		    ));

		$this->excel->setActiveSheetIndex(0)
		                            ->setCellValue('A4', 'Appointment Time')
		                            ->setCellValue('B4', 'Patient Name')
		                            ->setCellValue('C4', 'Phone Number')
		                            ->setCellValue('D4', 'Email Address');

		// $data = $this->reports_model->clientAllExcel();
		$sql = "SELECT B.`cmid`, B.`firstname`, B.`lastname`, A.`start_date`, A.`duration`,
				CONCAT(C.`firstname`,' ',C.`lastname`) as doctor, A.`appointment_reason`, A.`date_created`,
				B.`phone_number`, B.`email_address`
				FROM appointment A
				LEFT OUTER JOIN patients B ON B.`id` = A.`patient_id` 
				LEFT OUTER JOIN users C ON C.`user_id` = A.`assgined_doctor` 
				WHERE A.`assgined_doctor` = ".$doctor_id." AND A.`nStatus` = 1 AND 
				DATE_FORMAT(A.`start_date`, '%Y-%m-%d') = '".$year.'-'.$month.'-'.$day."' ORDER BY start_date ASC";
				// die($sql);
		$query = $this->db->query($sql);
		$data = $query->result();
		$i = 5;
		foreach($data as $row){
		    $this->excel->setActiveSheetIndex(0)
		    		->setCellValue('A'.$i, date('H:i A',strtotime($row->start_date)))
		            ->setCellValue('B'.$i, $row->firstname.' '.$row->lastname)
		            ->setCellValue('C'.$i, $row->phone_number)
		            ->setCellValue('D'.$i, $row->email_address);
		    $i++;
		}

		$filename = date("Ymd")."_AppointmentsDetails_DR".$doctorInfo->firstname.".xlsx";
		header('Content-Type: application/vnd.openxmlformats-officedocument.spreadsheetml.sheet'); //mime type
		header('Content-Disposition: attachment;filename="'.$filename.'"'); //tell browser what's the file name
		header('Cache-Control: max-age=0'); //no cache
		            
		//save it to Excel5 format (excel 2003 .XLS file), change this to 'Excel2007' (and adjust the filename extension, also the header mime type)
		//if you want to save it as .XLSX Excel 2007 format
		$objWriter = PHPExcel_IOFactory::createWriter($this->excel, 'Excel2007');  
		//force user to download the Excel file without writing it to server's HD
		$objWriter->save('php://output');
	}

	public function getDescApp($id)
	{
		$result = $this->appointment_model->getDetails($id);
		$cType = ($result->appointment_type == "1") ? "Initial Consult" : "Follow Up";
		$html = "<dl>
					<dt>Patient Name</dt>
					<dd>{$result->firstname} {$result->lastname}</dd>
					<dt>Duration</dt>
					<dd>{$result->duration} mins.</dd>
					<dt>Appointment Date</dt>
					<dd>".date('d/M/Y H:i A',strtotime($result->start_date))."</dd>
					<dt>Type</dt>
					<dd>{$cType}</dd>
					<dt>Assigned Doctor</dt>
					<dd>{$result->docFname} {$result->docLname}</dd>
					<dt>Created Date</dt>
					<dd>".date('d/M/Y H:i A',strtotime($result->date_created))."</dd>
					<dt>Remarks</dt>
					<dd>{$result->appointment_reason}</dd>
				</dl>";
		echo $html;
	}

	public function consult_notes($date, $doctor_id)
	{
		$this->load->library('zip');

		$year = explode('_', $date)[0];
		$month = explode('_', $date)[1];
		$day = explode('_', $date)[2];

		$doctorInfo = $this->user_model->getUserDetails('',$doctor_id);



		$query = $this->db->query("SELECT A.`patient_id`,B.`cmid`, B.`firstname`, B.`lastname`, A.`start_date`, A.`duration`,
				CONCAT(C.`firstname`,' ',C.`lastname`) as doctor, A.`appointment_reason`, A.`date_created`,
				B.`phone_number`, B.`email_address`
				FROM appointment A
				LEFT OUTER JOIN patients B ON B.`id` = A.`patient_id` 
				LEFT OUTER JOIN users C ON C.`user_id` = A.`assgined_doctor` 
				WHERE A.`assgined_doctor` = ".$doctor_id." AND A.`nStatus` = 1 AND A.`appointment_type` = 1 AND 
				DATE_FORMAT(A.`start_date`, '%Y-%m-%d') = '".$year.'-'.$month.'-'.$day."' ORDER BY start_date ASC");
		$data = $query->result();

		foreach($data as $row)
		{

			// ------------------------- CONSULT NOTES ------------------------------------
			$patient_details = $this->patients_model->getDetails($row->patient_id);
			$user_details = $this->user_model->getUserDetails('', $doctor_id);
			// $age = date_diff(date_create(), date_create($patient_details->dob));
			$age = General::computeAge($patient_details->dob);

			$consult_file = "./public/docx_template/consult_notes_".$doctor_id.".docx";
			$templateProcessor = new \PhpOffice\PhpWord\TemplateProcessor($consult_file);
			$firstname = strtolower($patient_details->firstname);
			$lastname = strtolower($patient_details->lastname);
			$templateProcessor->setValue('firstname', ucfirst($firstname));
			$templateProcessor->setValue('lastname', strtoupper($patient_details->lastname));
			$templateProcessor->setValue('fullname', ucfirst($firstname).' '.ucfirst($lastname));
			$templateProcessor->setValue('age', $age);
			$templateProcessor->setValue('idenage', $age.strtoupper(substr($patient_details->gender, 0, 1)));
			$templateProcessor->setValue('gender', $patient_details->gender);
			$templateProcessor->setValue('dob', strtoupper(date("d/M/Y",strtotime($patient_details->dob)))); //DD/MMM/YYYY
			$templateProcessor->setValue('today', strtoupper(date("d/M/Y"))); //DD/MMM/YYYY
			$templateProcessor->setValue('agegender', $age.'-year old ' . strtolower($patient_details->gender)); //DD/MMM/YYYY
			$templateProcessor->setValue('doctor_firstname', $user_details->firstname);
			$templateProcessor->setValue('doctor_lastname', $user_details->lastname);

			$filename = ucfirst($patient_details->lastname)."_".ucfirst($patient_details->firstname)."_".date('Ymd')."_Consult.docx";
			$templateProcessor->saveAs('./public/files/consults/'.$filename);

			// header('Content-Description: File Transfer');
		 //    header('Content-Type: application/octet-stream');
		 //    header('Content-Disposition: attachment; filename='.basename('./public/files/consults/'.$filename));
		 //    header('Expires: 0');
		 //    header('Cache-Control: must-revalidate');
		 //    header('Pragma: public');
		 //    header('Content-Length: ' . filesize('./public/files/consults/'.$filename));
		 //    readfile('./public/files/consults/'.$filename);

			$this->zip->read_file('./public/files/consults/'.$filename);

		    

			// ------------------------- CONSULT NOTES -------------------------------------

		}

		$zipfilename = date("Ymd")."_ConsultNotes_DR".$doctorInfo->firstname.".zip";
    	$this->zip->download($zipfilename);



	}


}
