<?php 

if ( ! defined('BASEPATH')) exit('No direct script access allowed');

require APPPATH.'controllers/General.php'; 

class Expiry_report extends General{

	function __construct() {
		parent::__construct();	

		if(General::isLoggedIN() === FALSE OR General::validateToken() === FALSE){
            redirect(base_url());    
        }

        General::ModAccess(__CLASS__);
        self::getPermissionAccess();
        
        $this->load->model('patients_model');
	}

	private function getPermissionAccess()
	{
		$this->data['printPermission'] = General::validatePermission('print_report', FALSE);
        $this->data['excelPermission'] = General::validatePermission('export_excel', FALSE);
        $this->data['pdfPermission'] = General::validatePermission('export_PDF', FALSE);
        $this->data['csvPermission'] = General::validatePermission('export_CSV', FALSE);
	}

	public function index()
	{
		$this->data['page_title'] = 'Reports';
		$this->data['page_sub_title'] = 'New Patients Report';
		$this->data['css'] = $this->assets_resources->getAssets('expiry_report_asset')['index']['css'];
		$this->data['js'] = $this->assets_resources->getAssets('expiry_report_asset')['index']['js'];
		$this->data['jsextend'] = $this->assets_resources->getAssets('expiry_report_asset')['index']['jsextend'];
		$this->data['file'] = $this->assets_resources->getAssets('expiry_report_asset')['index']['file'];
		$this->load->view("page", $this->data);
	}


	public function print_report()
	{

		$printOption = $this->input->post("printOption");

		if($printOption == "web_print")
		{
			$this->result_print();

			$log = 'Medical Expiry report has been printed by ' .General::getFullName();
			General::log('EXPIRY REPORT', 'PRINT', $log);

		}
		else if($printOption == "export_excel")
		{
			$this->result_excel();

			$log = 'Medical Expiry report has been exported to excel by ' .General::getFullName();
			General::log('EXPIRY REPORT', 'EXPORT EXCEL', $log);

		}
		else if($this->input->post('printOption') == "export_pdf")
		{
			$this->result_pdf();

			$log = 'Medical Expiry report has been exported to PDF by ' .General::getFullName();
			General::log('EXPIRY REPORT', 'EXPORT PDF', $log);

		}
		else if($this->input->post('printOption') == "export_csv")
		{
			$this->ExportCSV();

			$log = 'Medical Expiry report has been exported to CSV by ' .General::getFullName();
			General::log('EXPIRY REPORT', 'EXPORT CSV', $log);

		}

		
	}

	public function result_pdf()
	{
		$date_now = date('Y-m-d');
		$future_date = $this->input->post('future_date');
		$this->data['page_title'] = "Medical Expiry Report";
		$this->data['print'] = FALSE;
		$query = $this->db->query("SELECT B.`cmid`,B.`firstname`,B.`lastname`,A.`expiry_date`,A.`quantity`,
			A.`period_use`,A.`diagnosis_comments`,A.`thc`,A.`cbd`,A.`restrictions`,B.`email_address`
			FROM `patient_medical` A LEFT OUTER JOIN `patients` B ON B.`cmid` = A.`cmid`
			WHERE A.`expiry_date` BETWEEN '".$date_now."' AND '".$future_date."' AND A.`nStatus` = 1 ORDER BY A.`expiry_date`");
		$result = ($query->num_rows() > 0) ? $query->result() : FALSE;
		$this->data['rsDetails'] = $result;
		$html = $this->load->view('expiry_report/print', $this->data, true);
		$filename = "expiry_report_".date("YmdHis");
		$papersize = "A4";
		$orientation = $this->input->post('pdfOrientation');
        pdf_create($html, $filename, TRUE, $papersize, $orientation);
	}

	public function result_print()
	{
		$date_now = date('Y-m-d');
		$future_date = $this->input->post('future_date');
		$this->data['page_title'] = "Medical Expiry Report";
		$this->data['print'] = TRUE;
		$query = $this->db->query("SELECT B.`cmid`,B.`firstname`,B.`lastname`,A.`expiry_date`,A.`quantity`,
			A.`period_use`,A.`diagnosis_comments`,A.`thc`,A.`cbd`,A.`restrictions`,B.`email_address`
			FROM `patient_medical` A LEFT OUTER JOIN `patients` B ON B.`cmid` = A.`cmid`
			WHERE A.`expiry_date` BETWEEN '".$date_now."' AND '".$future_date."' AND A.`nStatus` = 1 ORDER BY A.`expiry_date`");
		$result = ($query->num_rows() > 0) ? $query->result() : FALSE;
		$this->data['rsDetails'] = $result;
		$this->load->view('expiry_report/print', $this->data);
	}

	public function ExportCSV()
	{
		$date_now = date('Y-m-d');
		$future_date = $this->input->post('future_date');
        $this->load->dbutil();
        $this->load->helper('file');
        $this->load->helper('download');
        $delimiter = ",";
        $newline = "\r\n";
        $filename = "patients_report_".date("YmdHis").'.csv';

		$result = $this->db->query("SELECT B.`cmid`,B.`firstname`,B.`lastname`,A.`expiry_date`,A.`quantity`,
			A.`period_use`,A.`diagnosis_comments`,A.`thc`,A.`cbd`,A.`restrictions`,B.`email_address`
			FROM `patient_medical` A LEFT OUTER JOIN `patients` B ON B.`cmid` = A.`cmid`
			WHERE A.`expiry_date` BETWEEN '".$date_now."' AND '".$future_date."' AND A.`nStatus` = 1 ORDER BY A.`expiry_date`");

        $data = $this->dbutil->csv_from_result($result, $delimiter, $newline);
        force_download($filename, $data);
	}


	public function result_excel()
	{
		//load our new PHPExcel library
		$this->load->library('excel');
		//activate worksheet number 1
		$this->excel->setActiveSheetIndex(0);
		//name the worksheet
		$this->excel->getActiveSheet()->setTitle('Patients Report');
		$this->excel->getProperties()->setCreator("Company Name")
									 ->setLastModifiedBy('Name of Modified')
									 ->setSubject("Patients Report Excel")
									 ->setTitle('Patients Report')
									 ->setDescription("List of Patients")
									 ->setCategory("Reports Generation");

		$this->excel->setActiveSheetIndex(0)
		                            ->setCellValue('A1', 'EXPIRY DATE')
		                            ->setCellValue('B1', 'DAYS LEFT')
		                            ->setCellValue('C1', 'CMID')
		                            ->setCellValue('D1', 'FIRST NAME')
		                            ->setCellValue('E1', 'LAST NAME')
		                            ->setCellValue('F1', 'QUANTITY')
		                            ->setCellValue('G1', 'PERIOD USE')
		                            ->setCellValue('H1', 'THC')
		                            ->setCellValue('I1', 'CBD')
		                            ->setCellValue('J1', 'REMARKS')
		                            ->setCellValue('K1', 'EMAIL');


		$date_now = date('Y-m-d');
		$future_date = $this->input->post('future_date');
		$query = $this->db->query("SELECT B.`cmid`,B.`firstname`,B.`lastname`,A.`expiry_date`,A.`quantity`,
			A.`period_use`,A.`diagnosis_comments`,A.`thc`,A.`cbd`,A.`restrictions`,B.`email_address`
			FROM `patient_medical` A LEFT OUTER JOIN `patients` B ON B.`cmid` = A.`cmid`
			WHERE A.`expiry_date` BETWEEN '".$date_now."' AND '".$future_date."' AND A.`nStatus` = 1 ORDER BY A.`expiry_date`");
		$result = ($query->num_rows() > 0) ? $query->result() : FALSE;
		$this->data['rsDetails'] = $result;
		$data = ($query->num_rows() > 0) ? $query->result() : FALSE;

		if($data)
		{
			$i = 2;
			foreach($data as $row){
				$dStart = new DateTime(date('Y-m-d'));
		   		$dEnd  = new DateTime($row->expiry_date);
		   		$dDiff = $dStart->diff($dEnd);

			    $this->excel->setActiveSheetIndex(0)
			            ->setCellValue('A'.$i, $row->expiry_date)
			            ->setCellValue('B'.$i, $dDiff->days)
			            ->setCellValue('C'.$i, $row->cmid)
			            ->setCellValue('D'.$i, $row->firstname)
			            ->setCellValue('E'.$i, $row->lastname)
			            ->setCellValue('F'.$i, $row->quantity)
			            ->setCellValue('G'.$i, $row->period_use)
			            ->setCellValue('H'.$i, $row->thc)
			            ->setCellValue('I'.$i, $row->cbd)
			            ->setCellValue('J'.$i, $row->diagnosis_comments)
			            ->setCellValue('K'.$i, $row->email_address);
			    $i++;
			}
		}

		

		$filename = "expiry_report_".date("YmdHis").'.xlsx';
		header('Content-Type: application/vnd.openxmlformats-officedocument.spreadsheetml.sheet'); //mime type
		header('Content-Disposition: attachment;filename="'.$filename.'"'); //tell browser what's the file name
		header('Cache-Control: max-age=0'); //no cache
		            
		//save it to Excel5 format (excel 2003 .XLS file), change this to 'Excel2007' (and adjust the filename extension, also the header mime type)
		//if you want to save it as .XLSX Excel 2007 format
		$objWriter = PHPExcel_IOFactory::createWriter($this->excel, 'Excel2007');  
		//force user to download the Excel file without writing it to server's HD
		$objWriter->save('php://output');
	}


	

	

}