<?php

defined('BASEPATH') OR exit('No direct script access allowed');

require APPPATH.'controllers/General.php'; 

class Export extends General {

	function __construct() {
		parent::__construct();	
		$this->load->model('general_model');
		$this->load->model('export_model');
		$this->load->model('designation_model');
		$this->load->model('bulletin_model');
		$this->load->model('department_model');
		$this->load->model('user_model');
		$this->load->model('group_model');
		$this->load->model('patients_model');
		$this->load->model('appointment_model');
		if(General::isLoggedIN() === FALSE OR General::validateToken() === FALSE){
            redirect(base_url());    
        }
	}

	public function print_page($cType, $module)
	{
		$page_title = ucfirst(str_replace("_", " ", $module)). ' Report';
		$this->data['page_title'] = $page_title;
		$this->data['print'] = TRUE;
		$isCSV = ($cType == "csv") ? TRUE : FALSE;

		switch ($module) {
			case 'designation':
				$rsDetails = $this->designation_model->getAllDesignation(0, 0, FALSE, TRUE, $isCSV);
				$columns = array('Designation','Description','Status');
				$fields = array('designation_name','designation_desc','nStatus');
				break;
			case 'department':
				$rsDetails = $this->department_model->getAllDepartment(0, 0, FALSE, TRUE, $isCSV);
				$columns = array('Department Name','Description','Status');
				$fields = array('department_name','department_desc','nStatus');
				break;
			case 'bulletin_board':
				$rsDetails = $this->bulletin_model->getAllData(0, 0, FALSE, TRUE, $isCSV);
				$columns = array('Subject','Message','Date Added','Added By','Post to Datshboard');
				$fields = array('subject','message','dDateAdded','addedBy','postToDashboard');
				break;
			case 'users':
				$rsDetails = $this->user_model->getUsers(0, 0, FALSE, TRUE, $isCSV);
				$columns = array('Name','Email Address','Phone','Group','Status');
				$fields = array('name','email','phone','group_name','nStatus');
				break;
			case 'user_group':
				$rsDetails = $this->group_model->getGroups(0, 0, FALSE, TRUE, $isCSV);
				$columns = array('Group Name','Group Description','Status');
				$fields = array('group_name','group_desc','nStatus');
				break;
			case 'patients':
				$rsDetails = $this->patients_model->getAll(0, 0, FALSE, TRUE, $isCSV);
				$columns = array('CMID','First Name','Last Name','Email Address','Date of Birth','Gender','Phone Number','Status');
				$fields = array('cmid','firstname','lastname','email_address','dob','gender','phone_number','nStatus');
				break;
			case 'appointment':
				$rsDetails = $this->appointment_model->getAll(0, 0, FALSE, TRUE, $isCSV);
				$columns = array('Appointment Start Date','Appointment End Date','CMID','Patient First Name','Patient Last Name','Assigned Doctor','Appointment Remarks','Created Date');
				$fields = array('start_date','end_date','cmid','firstname','lastname','assgined_doctor','appointment_reason','date_created');
				break;
			default:
				die('');
				break;
		}

		$this->data['rsDetails'] = $rsDetails;
		$this->data['columns'] = $columns;
		$this->data['fields'] = $fields;

		if($cType == "pdf")
		{
			$html = $this->load->view("export/print", $this->data, TRUE);
			$filename = $module.'-'.date("YmdHis");
			$papersize = "A4";
			$orientation = 'landscape'; // landscape or portrait
			pdf_create($html, $filename, TRUE, $papersize, $orientation);
		} 
		else if($cType == "excel")
		{
			self::printExcel($page_title, $rsDetails, $columns, $fields);
		} 
		else if($cType == "csv")
		{
			self::printCSV($page_title, $rsDetails, $columns, $fields);
		} 
		else if($cType == "print")
		{
			$this->load->view("export/print", $this->data);
		}
		
	}

	private function printCSV($page_title, $rsDetails, $columns, $fields)
	{
		$this->load->dbutil();
        $this->load->helper('file');
        $this->load->helper('download');
        $delimiter = ",";
        $newline = "\r\n";
        $filename = $page_title.".csv";

        $data = $this->dbutil->csv_from_result($rsDetails, $delimiter, $newline);
        force_download($filename, $data);
	}

	private function printExcel($page_title, $rsDetails, $columns, $fields)
	{
		$letter = array('A','B','C','D','E','F','G','H','I','J','K','L','M','N','O','P','Q','R','S','T','U','V','W','X','Y','Z');

		$this->load->library('excel');
		$this->excel->setActiveSheetIndex(0);
		$this->excel->getActiveSheet()->setTitle($page_title);
		$this->excel->getProperties()->setCreator(General::getCompanyInfo()->company_name)
									 ->setLastModifiedBy(General::getFullName())
									 ->setSubject($page_title . " Excel")
									 ->setTitle($page_title);
		$this->excel->getActiveSheet()->setCellValue('A1', General::getCompanyInfo()->company_name);
		$this->excel->getActiveSheet()->getStyle('A1')->getFont()->setSize(16);
		$this->excel->getActiveSheet()->mergeCells('A1:I1');
		$this->excel->getActiveSheet()->setCellValue('A2', $page_title);
		$this->excel->getActiveSheet()->getStyle('A2')->getFont()->setSize(12);
		$this->excel->getActiveSheet()->mergeCells('A2:I2');

		$maxCountCol = count($fields) - 1;
		$maxCol = $letter[$maxCountCol].'4';
		$this->excel->getActiveSheet()->getStyle('A4:'.$maxCol)->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);

		foreach(range('A4',$maxCol) as $columnID) {
		    $this->excel->getActiveSheet()->getColumnDimension($columnID)
		        ->setAutoSize(true);
		}

		$this->excel->getActiveSheet()->getStyle('A4:'.$maxCol)->getFill()->applyFromArray(array(
	        'type' => PHPExcel_Style_Fill::FILL_SOLID,
	        'startcolor' => array(
	             'rgb' => 'CCCCCC'
	        )
	    ));

		$c=0;
		foreach($columns as $col){
			$this->excel->setActiveSheetIndex(0)->setCellValue($letter[$c].'4', $col);
			$c++;
		}

		if($rsDetails)
		{
			$i = 5;
			foreach($rsDetails as $row)
			{
				for($countFields = 0; $countFields <= count($fields) - 1; $countFields++){
					if($fields[$countFields] == 'nStatus') {
						$valField = ($row->$fields[$countFields]) ? 'Active' : '<span style="color:red">InActive</span>';
					} else {
						$valField = $row->$fields[$countFields];
					}
					$this->excel->setActiveSheetIndex(0)->setCellValue($letter[$countFields].$i, $valField);
				}
				$i++;
			}
		}

		$filename = $page_title.date("YmdHis").'.xlsx';
		header('Content-Type: application/vnd.openxmlformats-officedocument.spreadsheetml.sheet'); 
		header('Content-Disposition: attachment;filename="'.$filename.'"'); 
		header('Cache-Control: max-age=0'); 

		$objWriter = PHPExcel_IOFactory::createWriter($this->excel, 'Excel2007');  
		$objWriter->save('php://output');

	}


	
}
