<?php 

if ( ! defined('BASEPATH')) exit('No direct script access allowed');

class General extends CI_Controller{

	public $timezone, $language, $currency;
	private $menu_id;

    const SHORT_SYS_TITLE = 'Marcos Clinic';
	const SYSTEM_TITLE = 'Marcos Clinic';
	const FOOTER_TITLE = ' &copy; Marcos Clinic';
    const MALE = 1;
    const FEMALE = 0;
    const EMAIL_FROM = 'no-reply@jaysonsarino.com';
    const EMAIL_FROM_NAME = 'Marcos Clinic';

	function __construct()
	{
		parent::__construct();	
		$this->load->model('general_model');
		$this->load->model('sidebar_model');
        $this->load->model('user_model');
        $this->load->model('sys_param_model');
        $this->load->model('company_information_model');
        $this->load->model('category_model');
        $this->load->model('patients_model');
        $this->load->model('appointment_model');
		
		// Load general parameters
		self::loadData();
		
	}

    public function deleteApp($id)
    {
        $rsDetails = $this->appointment_model->deleteApp($id);
        $data_arr = array('success' => TRUE, 'message' => 'Appointment has been deleted');
        $log = 'New Appointment: with id#:<strong>'.$id.'</strong> has been deleted by ' .General::getFullName();
        General::log('APPOINTMENTS', 'DELETE', $log);
    }

    public function doneApp($id)
    {
        $rsDetails = $this->appointment_model->doneApp($id);
        $data_arr = array('success' => TRUE, 'message' => 'Appointment has been marked as done');
        $log = 'New Appointment: with id#:<strong>'.$id.'</strong> has been marked as done by ' .General::getFullName();
        General::log('APPOINTMENTS', 'UPDATE', $log);
    }


    public function getAppointmentDetails($id)
    {
        General::addJSONResponseHeader();
        $rsDetails = $this->appointment_model->getDetails($id);
        $rsDetails->date = date('Y-m-d', strtotime($rsDetails->start_date));
        die(json_encode($rsDetails));
    }

	private function loadData()
	{
        // reset pageclosed session
        $this->session->set_userdata('pageclosed', FALSE);
		// Get default timezone settings
		date_default_timezone_set('Asia/Manila');

		// Get default language settings
		$this->language = self::getDefaultLanguage();
		$this->data['curr_lang'] = $this->language;
		$this->lang->load($this->language, $this->language);
        $this->data['autocmid'] = self::getCMID() + 1;

		// Load all languages option
        $this->data['lang_opt'] = self::getSystemOptions('lang', $this->language);

        // Is user allow to change theme
        $this->data['isAllowChangeTheme'] = $this->general_model->getSystemSettings('theme_allowed')->settings_value;

        // Get active currency symbol
        $this->currency = $this->general_model->getSystemSettings('currency')->settings_value;
        $this->data['currency'] = $this->currency;

        $this->data['system_title'] = self::SYSTEM_TITLE;
		$this->data['short_system_title'] = self::SHORT_SYS_TITLE;
		$this->data['footer_title'] = date('Y') . self::FOOTER_TITLE;

        $companyInfo = $this->company_information_model->getDataInfo('1');
        $this->data['companyInfo'] = $this->company_information_model->getDataInfo('1');

		// Load parent menu
		if ($this->session->userdata('isLoggedIN') === TRUE) {
			$this->data['parent_menus'] = self::getParentMenus();
            $this->data['fullname'] = self::getFirstName();
			$this->data['userDetails'] = self::getUserProfile();

            // Get user theme
            $this->data['theme'] = self::getDefaultTheme();

            $this->data['addDesignation'] = self::validateQuickMenu('add', 3);
            $this->data['addDepartment'] = self::validateQuickMenu('add', 2);
            $this->data['addUser'] = self::validateQuickMenu('add', 5);
            $this->data['addGroup'] = self::validateQuickMenu('add', 6);
            $this->data['addBulletin'] = self::validateQuickMenu('add', 25);

            $this->data['patientList'] = $this->patients_model->getAll('', '', '', TRUE);
            $this->data['doctorList'] = $this->user_model->getUserByGroup(5);
		}
		
	}

    public function add_appointment()
    {
        General::addJSONResponseHeader();

        try {
            $this->form_validation->set_rules("patient_id","Patient","trim|xss_clean|required");
            $this->form_validation->set_rules("date","Appointment Date","trim|xss_clean|required");
            $this->form_validation->set_rules("assgined_doctor","Assigned Doctor","trim|xss_clean|required");
            $this->form_validation->set_rules("appointment_reason","Appointment Remarks","trim|xss_clean");
            $this->form_validation->set_rules("appointment_type","Appointment Type","trim|xss_clean|required");
            $this->form_validation->set_error_delimiters('', '');

            if($this->form_validation->run())
            {   
                $result = $this->appointment_model->save('insert');
                if($result) 
                {
                    // Record activity log
                    $log = 'New Appointment: with id#:<strong>'.$result.'</strong> has been added by ' .General::getFullName();
                    General::log('APPOINTMENTS', 'INSERT', $log);

                    $data_arr = array('success' => TRUE, 'message' => 'Appointment has been saved', 'formaction' => 'add_appointment');

                } else {
                    $data_arr = array('success' => FALSE, 'message' => 'Their\'s an error in saving patient.');
                }
            }
            else
            {
                $data_arr = array('success' => FALSE, 'message' => validation_errors());
            }
        } catch(Exception $e) {
            $data_arr = array("success" => FALSE, "message" => $e->getMessage());
        }

        die(json_encode($data_arr));
    }

    public function getCompanyInfo() {
        return $this->company_information_model->getDataInfo('1');
    }

    private function validateQuickMenu($action, $menu_id) {
        $sql = $this->db->query("SELECT `group_id` FROM `group_permission` WHERE `menu_id` = " . $menu_id . " AND `permission_name` = '" . $action . "'");
        $pri = unserialize($sql->row()->group_id);
        $result = in_array($this->session->userdata('group_id'), $pri);
        return $result;
    }

	public function addJSONResponseHeader() {
        header("Cache-Control: no-store, no-cache, must-revalidate, max-age=0");
        header("Cache-Control: post-check=0, pre-check=0", false);
        header("Pragma: no-cache");
        header("Content-Type: application/json");
    }

	private function getDefaultTimeZone() {
		$this->timezone = $this->general_model->getSystemSettings('default_timezone')->settings_value;
		return ($this->timezone) ? $this->timezone : 'Asia/Manila';
	}

	private function getDefaultLanguage() {
		return $this->general_model->getSystemSettings('lang')->settings_value;
	}

    public function getCMID() {
        return $this->general_model->getSystemSettings('CMID')->settings_value;
    }

    public function getCMIDImport() {
        return $this->general_model->getSystemSettings('CMID')->settings_value;
    }

    private function getDefaultTheme() {
        return $this->user_model->getUserDetails(NULL, $this->session->userdata('user_id'))->theme;
    }

	public function getSystemOptions($cCode, $where_not = NULL) {
		return $this->general_model->getSystemOptions($cCode, $where_not);
	}

	public function setLang($lang) {
		header("Content-Type: application/json");
		
		try {
			$result = $this->general_model->setLang($lang);
			if($result === TRUE) {
				$data_arr = array("success" => TRUE, "message" => 'Language has been changed.');
			} else {
				$data_arr = array("success" => FALSE, "message" => 'Their\'s an error in changing language.');
			}
		} catch(Exception $e) {
			$data_arr = array("success" => FALSE, "message" => $e->getMessage());
		}

        die(json_encode($data_arr));
	}

	public function generateRandomString($length) {
        $characters = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
        $charactersLength = strlen($characters);
        $randomString = '';
        for ($i = 0; $i < $length; $i++) {
            $randomString .= $characters[rand(0, $charactersLength - 1)];
        }
        return $randomString;
    }

    public function generateRandomNumber($length) {
        $characters = '0123456789';
        $charactersLength = strlen($characters);
        $randomString = '';
        for ($i = 0; $i < $length; $i++) {
            $randomString .= $characters[rand(0, $charactersLength - 1)];
        }
        return $randomString;
    }

    public function log($module, $event, $log) {
    	$this->general_model->log($module, $event, $log);
    }

    public function getFullName() {
    	$rsUserDetails = $this->user_model->getUserDetails($this->session->userdata('username'));
    	return $rsUserDetails->firstname.' '.$rsUserDetails->lastname;
    }

    public function getFirstName() {
    	$rsUserDetails = $this->user_model->getUserDetails($this->session->userdata('username'));
    	return $rsUserDetails->firstname;
    }

    public function getLastName() {
        $rsUserDetails = $this->user_model->getUserDetails($this->session->userdata('username'));
        return $rsUserDetails->lastname;
    }

    public function getUserProfile() {
        $rsUserDetails = $this->user_model->getUserDetails($this->session->userdata('username'));
        return $rsUserDetails;
    }

    public function getEmailAddress() {
    	$rsUserDetails = $this->user_model->getUserDetails($this->session->userdata('username'));
    	return $rsUserDetails->email;
    }

    public function getParentMenus() {
    	$privileges = unserialize(self::getPrivileges());
    	return $this->general_model->getParentMenus($privileges);
    }


    public function processAction($table, $where_field)
    {
        if (isset($_REQUEST['actionType']) && $_REQUEST['actionType'] == 'group_action') {
            if ($_REQUEST['actionName'] == 'Delete') {
                $ids = $_REQUEST['id'];
                if (is_array($ids)) {
                    foreach ($ids as $id) {
                        self::delete($table, $where_field, $id);
                    }
                }
            }
            if ($_REQUEST['actionName'] == 'DeActivate') {
                $ids = $_REQUEST['id'];
                if (is_array($ids)) {
                    foreach ($ids as $id) {
                        self::deactivate($table, $where_field, $id);
                    }
                }
            }
            if ($_REQUEST['actionName'] == 'Activate') {
                $ids = $_REQUEST['id'];
                if (is_array($ids)) {
                    foreach ($ids as $id) {
                        self::activate($table, $where_field, $id);
                    }
                }
            }
            if ($_REQUEST['actionName'] == 'Done') {
                $ids = $_REQUEST['id'];
                if (is_array($ids)) {
                    foreach ($ids as $id) {
                        self::done($table, $where_field, $id);
                    }
                }
            }
        }


    }

    public function delete($table, $where_field, $id)
	{
        $rsDetails = $this->general_model->getDeleteName($table, $where_field, NULL, $id);
		$result = $this->general_model->delete($table, $where_field, $id);
        if($result) {
            // Insert Log Activity
            $arrMod = $this->app_notifications->logs($table, $rsDetails);
            $log = 'New '.$arrMod['moduleName'].': <strong>'.$arrMod['fieldName'].'</strong> has been <strong>Deleted</strong> by ' .self::getFullName();
            self::log($arrMod['module'], 'DELETED', $log);
            return TRUE;
        } else {
            return FALSE;
        }
	}	

    public function deactivate($table, $where_field, $id)
    {
        $rsDetails = $this->general_model->getDeleteName($table, $where_field, NULL, $id);
        $result = $this->general_model->deactivate($table, $where_field, $id);
        if($result) {
            // Insert Log Activity
            $arrMod = $this->app_notifications->logs($table, $rsDetails);
            $log = 'New '.$arrMod['moduleName'].': <strong>'.$arrMod['fieldName'].'</strong> has been <strong>Deactivated</strong> by ' .self::getFullName();
            self::log($arrMod['module'], 'DEACTIVATE', $log);
            return TRUE;
        } else {
            return FALSE;
        }
    }

    public function activate($table, $where_field, $id)
    {
        $rsDetails = $this->general_model->getDeleteName($table, $where_field, NULL, $id);
        $result = $this->general_model->activate($table, $where_field, $id);
        if($result) {
            // Insert Log Activity
            $arrMod = $this->app_notifications->logs($table, $rsDetails);
            $log = 'New '.$arrMod['moduleName'].': <strong>'.$arrMod['fieldName'].'</strong> has been <strong>Activated</strong> by ' .self::getFullName();
            self::log($arrMod['module'], 'ACTIVATE', $log);
            return TRUE;
        } else {
            return FALSE;
        }
    }

    public function done($table, $where_field, $id)
    {
        $rsDetails = $this->general_model->getDeleteName($table, $where_field, NULL, $id);
        $result = $this->general_model->done($table, $where_field, $id);
        if($result) {
            // Insert Log Activity
            $arrMod = $this->app_notifications->logs($table, $rsDetails);
            $log = 'New '.$arrMod['moduleName'].': <strong>'.$arrMod['fieldName'].'</strong> marked as <strong>Done</strong> by ' .self::getFullName();
            self::log($arrMod['module'], 'DONE', $log);
            return TRUE;
        } else {
            return FALSE;
        }
    }

	// Validate user session
	public function isLoggedIN()
	{
		return ($this->session->userdata('isLoggedIN') === TRUE) ? TRUE : FALSE;
    }

    // Validate user token and session if match to user table
    public function validateToken()
	{
		return $this->general_model->validateToken();
    }

    public function ModAccess($classname)
    {
    	$query = $this->db->query("SELECT * FROM `menu` WHERE `menu_link` = '" . strtolower($classname) . "'");
    	$this->menu_id = $query->row()->menu_id;
    	$parent_id = $query->row()->parent_id;

    	$sql = $this->db->query("SELECT `privileges` FROM `user_group` WHERE `group_id` = " . $this->session->userdata('group_id'));
    	$pri = unserialize($sql->row()->privileges);
    	if(!in_array($this->menu_id, $pri)) {
    		redirect(base_url('page_error/code/401'));
    	} else {
    		$this->session->set_userdata(array(
    			'parent_id'		=>		$parent_id,
    			'menu_id'		=>		$this->menu_id
    		));
    	} 
    }

    public function validatePermission($action, $redirect = TRUE)
    {
    	$sql = $this->db->query("SELECT `group_id` FROM `group_permission` WHERE `menu_id` = " . $this->menu_id . " AND `permission_name` = '" . $action . "'");
    	$pri = unserialize($sql->row()->group_id);
    	if(!in_array($this->session->userdata('group_id'), $pri)) {
    		if($redirect === TRUE) {
    			redirect(base_url('page_error/code/401'));
    		} else {
    			return TRUE;
    		}
    	} else {
    		return FALSE;
    	}
    }

    public function getPrivileges()
    {
    	$sql = $this->db->query("SELECT `privileges` FROM `user_group` WHERE `group_id` = " . $this->session->userdata('group_id'));
    	return $sql->row()->privileges;
    }

    public function validateFields($data = array())
    {
    	$table = $data['table'];
    	$where_not = '';

    	if(is_array($data['fields'])) {

    		$fields = array();
    		foreach($data['fields'] as $key => $val) {
    			$fields[] = $key . " =  '" . $val . "'";
    		}

    		$where =  implode(" AND ", $fields);

    	} 

    	if(is_array($data['edit'])) {
    		$where_not = " AND ". $data['edit'][0] . " != " . $data['edit'][1];
    	}

    	$sql = "SELECT * FROM {$table} WHERE " . $where . $where_not;
		$query = $this->db->query($sql);
		if($query->num_rows() > 0) {
			return TRUE;
		} else {
			return FALSE;
		}	
    }

    public function sendsms() {
        $this->load->library('twilio');

        $from = '';
        $to = '+639987906675';
        $message = 'This is a test...';

        $response = $this->twilio->sms($from, $to, $message);


        if($response->IsError)
            echo 'Error: ' . $response->ErrorMessage;
        else
            echo 'Sent message to ' . $to;
    }

    public function resize_img($path, $width, $height)
    {
        $config['image_library'] = 'gd2';
        $config['source_image'] = FCPATH . $path;
        $config['create_thumb'] = TRUE;
        $config['maintain_ratio'] = TRUE;
        $config['width']         = $width;
        $config['height']       = $height;

        $this->load->library('image_lib', $config);

        $result = $this->image_lib->resize();
        return $result;
    }

    public function changeTheme($color) {
        $this->db->query("UPDATE `users` SET `theme` = '".$color."' WHERE `user_id` = " . $this->session->userdata('user_id'));
    }

    public function save_new_quick()
    {   
        self::addJSONResponseHeader();
        try {
            $ffields = $_POST['foreign_fields'];
            $foreign_table = $_POST['foreign_table'];
            $table = array_keys($_POST)[2];
            $fields = $_POST[$table];
            $result = $this->general_model->save_new_quick($table, $fields);
            if($result) {
                if(!empty($foreign_table)) {
                    // For department and designation table
                    // use if table has no foreign key but not parameters
                    if($foreign_table == "noforeign") 
                    {
                        $rs = $this->general_model->getDetailsQuick($table, $ffields, $result);
                    } 
                    else {
                        $ftable = explode('|', $foreign_table);
                        foreach($ftable as $ctable) {
                            $this->general_model->fsave_new_quick($ctable, $result, $ffields);
                        }
                        $rs = $this->general_model->getDetailsQuick($table, $ffields, $result);
                    }
                    
                } else { // else system parameters
                    $rs = $this->sys_param_model->getParamDetails($result);
                }
                if($table == 'organization_information') { $field_name = $rs->organization_name; }
                if($table == 'supplier_information') { $field_name = $rs->supplier_name; }
                if($table == 'system_parameters') { $field_name = $rs->param_value; }
                if($table == 'department') { $field_name = $rs->department_name; }
                if($table == 'designation') { $field_name = $rs->designation_name; }
                if($table == 'contact_information') { $field_name = $rs->firstname.' '.$rs->lastname; }
                if($table == 'opportunity') { $field_name = $rs->opportunity_name; }
                if($table == 'projects') { $field_name = $rs->name; }
                if($table == 'subcontractor') { $field_name = $rs->name; }
                $data_arr = array('success'=>TRUE,'last_id'=>$result,'field_name'=>$field_name,'message'=>$field_name . ' has been added.');
            } else {
                $data_arr = array('success'=> FALSE,'last_id'=>0,'message'=>$field_name . ' error in saving.');
            } 
        } catch(Exception $e) {
            $data_arr = array("success" => FALSE, "message" => $e->getMessage());
        }
        die(json_encode($data_arr));
    }

    public function computeAge($bdate)
    {
        $birthDate = "09/27/1989";
        $birthDate = date("m/d/Y",strtotime($bdate));
        $birthDate = explode("/", $birthDate);
        $age = (date("md", date("U", mktime(0, 0, 0, $birthDate[0], $birthDate[1], $birthDate[2]))) > date("md")
        ? ((date("Y") - $birthDate[2]) - 1)
        : (date("Y") - $birthDate[2]));
        return $age;
    }

    public function edit_appointment()
    {
        General::addJSONResponseHeader();

        try {
            $this->form_validation->set_rules("date","Appointment Date","trim|xss_clean|required");
            $this->form_validation->set_rules("appointment_reason","Appointment Remarks","trim|xss_clean");
            $this->form_validation->set_rules("appointment_type","Appointment Type","trim|xss_clean|required");
            $this->form_validation->set_error_delimiters('', '');

            if($this->form_validation->run())
            {   
                $result = $this->appointment_model->save('update');
                if($result) 
                {
                    // Record activity log
                    $log = 'New Appointment: with id#:<strong>'.$result.'</strong> has been updated by ' .General::getFullName();
                    General::log('APPOINTMENTS', 'UPDATE', $log);

                    $data_arr = array('success' => TRUE, 'message' => 'Appointment has been updated', 'formaction' => 'edit_appointment');

                } else {
                    $data_arr = array('success' => FALSE, 'message' => 'Their\'s an error in updating patient.');
                }
            }
            else
            {
                $data_arr = array('success' => FALSE, 'message' => validation_errors());
            }
        } catch(Exception $e) {
            $data_arr = array("success" => FALSE, "message" => $e->getMessage());
        }

        die(json_encode($data_arr));
    }


}