<?php

defined('BASEPATH') OR exit('No direct script access allowed');

require APPPATH.'controllers/General.php'; 


class Login extends General {

	function __construct(){

		parent::__construct();	
		$this->load->model('login_model');
		$this->load->model('user_model');
	}

	public function index()
	{
		$this->data['page_title'] = $this->lang->line('login_button');
		$this->data['session_active'] = false;
		if($this->session->userdata('isLoggedIN') AND General::isLoggedIN() === TRUE OR General::validateToken() === TRUE){
			$this->data['session_active'] = true;
			$this->data['userDetails'] = $this->user_model->getUserDetails('', $this->session->userdata('user_id'));
		}
		$this->load->view("login",$this->data);

	}

	public function validateLogin()
	{
		General::addJSONResponseHeader();

		try {

			$this->form_validation->set_rules("username","Username","trim|xss_clean|required");
			$this->form_validation->set_rules("password","Password","trim|xss_clean|required");
			$this->form_validation->set_error_delimiters("<br />","<br />");

			if ($this->form_validation->run()) 
			{
				// Get user password by username
				$rsUserDetails = $this->user_model->getUserDetails($this->input->post('username'));

				if ($rsUserDetails) 
				{
					// Validate password using phppass library
					$result = $this->password->check_password($this->input->post('password'), $rsUserDetails->password);

					if ($result) 
					{
						// Generate tokenID and sessionID 
						$tokenID = General::generateRandomString(50);
						$sessionID = General::generateRandomString(50);

						// Set session value
						$this->session->set_userdata('user_id', $rsUserDetails->user_id);
						$this->session->set_userdata('username', $rsUserDetails->username);
						$this->session->set_userdata('isLoggedIN', TRUE);
						$this->session->set_userdata('group_id', $rsUserDetails->group_id);
						$this->session->set_userdata('token_id', $tokenID);
						$this->session->set_userdata('session_id', $sessionID);

						// Update sessionID and tokenID
						$this->login_model->updateToken($tokenID, $sessionID);

						// Record activity log
						$log = General::getFullName() . ' has been Login.';
						General::log('LOGIN', 'LOGIN', $log);

						$data_arr = array("success" => TRUE, "message" => $this->lang->line('login_access_granted'), "group_id" => $rsUserDetails->group_id);
					} 
					else 
					{
						$data_arr = array("success" => FALSE, "message" => $this->lang->line('login_invalid_login'));
					}
				} 
				else 
				{
					$data_arr = array("success" => FALSE, "message" => $this->lang->line('login_invalid_username'));
				}

			} 
			else 
			{
				$data_arr = array("success" => FALSE, "message" => $this->lang->line('login_invalid_login'));
			}
		
		} catch(Exception $e) {
			$data_arr = array("success" => FALSE, "message" => $e->getMessage());
		}

        die(json_encode($data_arr));
	}

	public function test()
	{
		$this->load->view('email_template/new_user', $this->data);
	}

	public function validateEmailAddress()
	{
		General::addJSONResponseHeader();

		try
		{
			// Generate random string forgot password link
			$fpTokenID = General::generateRandomString(50);

			// Validate email address
			$result = $this->login_model->validateEmailAddress();

			// Email Address found
			if($result)
			{
				// Update user fpTokenID
				$this->login_model->updateUserFPToken($fpTokenID);

				// Url token ID
				$urltoken = $result->user_id."xyzabc".$fpTokenID;

				$this->data['urltoken'] = $urltoken;
				$this->data['firstname'] = $result->firstname;
				$body_email = $this->load->view('email_template/forgot_password', $this->data, true);
				$this->email->set_mailtype("html");
				$this->email->from('no-reply@yourreoffice.com', 'Admin Development Corporation');
				$this->email->to($this->input->post('email'));
				$this->email->subject(General::SHORT_SYS_TITLE . ' - Reset your Password');
				$this->email->message($body_email);
				$result = $this->email->send();

				if ($result) 
				{
					$data_arr['success'] = TRUE;
					$data_arr['message'] = $this->lang->line('login_forgot_password_msg') . ' '.$this->input->post('email'). ' ' . $this->lang->line('login_forgot_password_msgc');
				}
				else
				{
					$data_arr['success'] = FALSE;
					$data_arr['message'] = $this->lang->line('login_forgot_password_mail_not_send');
				}

			}
			else 
			{
				$data_arr['success'] = FALSE;
				$data_arr['message'] = $this->lang->line('login_forgot_password_mail_not_found');
			}
		} catch(Exception $e) {
			$data_arr = array("success" => FALSE, "message" => $e->getMessage());
		}
		
		die(json_encode($data_arr));
	}

	public function resetpassword()
	{
		General::addJSONResponseHeader();

		try {
			$this->form_validation->set_rules("newpassword","New Password","trim|required|min_length[6]");
			$this->form_validation->set_rules("confirmpassword","Confirm Password","trim|required|matches[newpassword]");
			$this->form_validation->set_error_delimiters('', '');

			if($this->form_validation->run())
			{	
				$hashPassword = $this->password->hash($this->input->post('newpassword'));
				$rsResult = $this->user_model->change_password($hashPassword, TRUE);
				if($rsResult){

					$rsUser = $this->user_model->getUserDetails('', $this->input->post('user_id'));
					$this->data['newpassword'] = $this->input->post('newpassword');
					$body_email = $this->load->view('email_template/password_changed', $this->data, true);
					$this->email->set_mailtype("html");
					$this->email->from('no-reply@yourreoffice.com', 'Admin Development Corporation');
					$this->email->to($rsUser->email);
					$this->email->subject(General::SHORT_SYS_TITLE . ' - Password Changed');
					$this->email->message($body_email);
					$result = $this->email->send();

					$data_arr = array('success' => TRUE, 'message' => 'Your have successfully changed your password.');
				} else {
					$data_arr = array('success' => FALSE, 'message' => 'Their\'s an error in changing your password.');
				}	
			}
			else
			{
				$data_arr = array('success' => FALSE, 'message' => validation_errors());
			}
		} catch(Exception $e) {
			$data_arr = array("success" => FALSE, "message" => $e->getMessage());
		}

		die(json_encode($data_arr));
	}

	public function insert_patient_id()
	{
		$data_arr = 'work';

		die($data_arr);

		echo json_encode($data_arr);
	}

}
