<?php

defined('BASEPATH') OR exit('No direct script access allowed');

require APPPATH.'controllers/General.php'; 


class Manage_department extends General {

	private $editPermission;

	function __construct() {
		parent::__construct();	

		if(General::isLoggedIN() === FALSE OR General::validateToken() === FALSE){
            redirect(base_url());    
        }

        General::ModAccess(__CLASS__);
        self::getPermissionAccess();
        
        $this->load->model('department_model');
	}

	private function getPermissionAccess()
	{
		$this->data['addPermission'] = General::validatePermission('add', FALSE);
        $this->data['deletePermission'] = General::validatePermission('delete', FALSE);
        $this->data['deactivatePermission'] = General::validatePermission('deactivate', FALSE);
        $this->data['activatePermission'] = General::validatePermission('activate', FALSE);
        $this->editPermission = General::validatePermission('edit', FALSE);
        $this->data['editPermission'] = $this->editPermission;
	}

	public function index()
	{
		$this->data['page_title'] = $this->lang->line('manage_department_title');
		$this->data['page_sub_title'] = $this->lang->line('manage_department_subtitle');
		$this->data['css'] = $this->assets_resources->getAssets('manage_department_asset')['index']['css'];
		$this->data['js'] = $this->assets_resources->getAssets('manage_department_asset')['index']['js'];
		$this->data['jsextend'] = $this->assets_resources->getAssets('manage_department_asset')['index']['jsextend'];
		$this->data['file'] = $this->assets_resources->getAssets('manage_department_asset')['index']['file'];
		$this->data['dt'] = array('datatable' => TRUE, 'path' => __CLASS__ . '/manage', 'sort' => array(1, 'asc'));
		$this->load->view("page", $this->data);
	}

	public function manage()
	{
		if (isset($_REQUEST['actionType']) && $_REQUEST['actionType'] == 'group_action') 
		{
			General::processAction('department', 'department_id');
			self::datatable();
		} else if(isset($_REQUEST['action']) && $_REQUEST['action'] == 'filter') {
			self::datatable($search = TRUE);
		} else {
			self::datatable();
		}
	}

	private function datatable($search = FALSE)
	{
		$result = $this->department_model->getAllDepartment('', '', $search, TRUE);
		$iTotalRecords = count($result);
		$iDisplayLength = intval($_REQUEST['length']);
		$iDisplayLength = $iDisplayLength < 0 ? $iTotalRecords : $iDisplayLength; 
		$iDisplayStart = intval($_REQUEST['start']);
		$sEcho = intval($_REQUEST['draw']);

		$records = array();

		$end = $iDisplayStart + $iDisplayLength;
		$end = $end > $iTotalRecords ? $iTotalRecords : $end;

		$rsData = $this->department_model->getAllDepartment($end, $iDisplayStart, $search, TRUE);

		if($rsData) {
			foreach ($rsData as $rsData) 
			{
				$hideEdit = ($this->editPermission) ? "hide" : "";

				$status = ($rsData->nStatus) ? "success" : "danger";
				$statusName = ($rsData->nStatus) ? "Active" : "InActive";
				$records["data"][] = array(
					'<label class="mt-checkbox mt-checkbox-single mt-checkbox-outline"><input name="id[]" type="checkbox" class="checkboxes" value="'.$rsData->department_id.'"/><span></span></label>',
					$rsData->department_name,
					$rsData->department_desc,
					'<span class="label label-sm label-'.$status.'">'.$statusName.'</span>',
					'<a href="'.base_url(__CLASS__ . "/edit/" . $rsData->department_id).'" class="btn btn-xs btn-outline grey-salsa"><i class="fa fa-edit"></i> Edit</a>'
				);
			}
		} else {
			$records["data"][] = array('','','','','');
		}

		$records["draw"] = $sEcho;
		$records["recordsTotal"] = $iTotalRecords;
		$records["recordsFiltered"] = $iTotalRecords;

		echo json_encode($records);
	}

	public function edit($id = '')
	{
		General::validatePermission(__FUNCTION__);

		if(isset($_POST['department_name']) AND $_POST['department_name'] != '') 
		{
			self::editsave();
		} 
		else 
		{
			$id = (int) $id;
			$this->data['page_title'] = $this->lang->line('manage_dept_edit');
			$this->data['page_sub_title'] = $this->lang->line('manage_department_title');
			$this->data['css'] = $this->assets_resources->getAssets('manage_department_asset')['add']['css'];
			$this->data['js'] = $this->assets_resources->getAssets('manage_department_asset')['add']['js'];
			$this->data['jsextend'] = $this->assets_resources->getAssets('manage_department_asset')['add']['jsextend'];
			$this->data['file'] = DIRECTORY_SEPARATOR . strtolower(__CLASS__) . DIRECTORY_SEPARATOR . __FUNCTION__ . '.php';
			$this->data['rsDetails'] = $this->department_model->getDepartmentDetails($id);
			$this->load->view("page", $this->data);
		}
	}

	public function editsave()
	{
		General::addJSONResponseHeader();

		try {
			$this->form_validation->set_rules("department_name","Department Name","trim|xss_clean|required|callback_validate_department");
			$this->form_validation->set_rules("department_desc","Description","trim|xss_clean|required");
			$this->form_validation->set_error_delimiters('', '');

			if($this->form_validation->run())
			{	
				$result = $this->department_model->save('update');
				if($result) 
				{
					// Record activity log
					$log = 'New Department: <strong>'. $this->input->post('department_name') . '</strong> has been updated by ' .General::getFullName();
					General::log('MANAGE DEPARTMENT', 'UPDATE', $log);

					$data_arr = array('success' => TRUE, 'message' => $this->lang->line('manage_department_successmsg'));

				} else {
					$data_arr = array('success' => FALSE, 'message' => $this->lang->line('manage_department_errormsg'));
				}
			}
			else
			{
				$data_arr = array('success' => FALSE, 'message' => validation_errors());
			}
		} catch(Exception $e) {
			$data_arr = array("success" => FALSE, "message" => $e->getMessage());
		}

		die(json_encode($data_arr));
	}

	public function add()
	{
	 	General::validatePermission(__FUNCTION__);

		if(isset($_POST['department_name']) AND $_POST['department_name'] != '') 
		{
			self::save();
		} 
		else 
		{
			$this->data['page_title'] = $this->lang->line('manage_department_addnew');
			$this->data['page_sub_title'] = $this->lang->line('manage_department_title');
			$this->data['css'] = $this->assets_resources->getAssets('manage_department_asset')['add']['css'];
			$this->data['js'] = $this->assets_resources->getAssets('manage_department_asset')['add']['js'];
			$this->data['jsextend'] = $this->assets_resources->getAssets('manage_department_asset')['add']['jsextend'];
			$this->data['file'] = $this->assets_resources->getAssets('manage_department_asset')['add']['file'];
			$this->load->view("page", $this->data);
		}
	}

	public function validate_department()
	{
		$data = array(
				'edit'		=>	(isset($_POST['department_id']) AND $_POST['department_id'] != '') ? array('department_id',$this->input->post('department_id')) : FALSE,
				'table'		=>	'department',
				'fields'	=>	array('department_name' => $this->input->post('department_name')),
		);
		if(General::validateFields($data) === TRUE) {
			$this->form_validation->set_message( __FUNCTION__ ,'Department Already Exists.');
			return FALSE;
		} else {
			return TRUE;
		}
	}

	public function save()
	{
		General::addJSONResponseHeader();

		try {
			$this->form_validation->set_rules("department_name","Department Name","trim|xss_clean|required|callback_validate_department");
			$this->form_validation->set_rules("department_desc","Description","trim|xss_clean|required");
			$this->form_validation->set_error_delimiters('', '');

			if($this->form_validation->run())
			{	
				$result = $this->department_model->save('insert');
				if($result) 
				{
					// Record activity log
					$log = 'New Department: <strong>'. $this->input->post('department_name') . '</strong> has been added by ' .General::getFullName();
					General::log('MANAGE DEPARTMENT', 'INSERT', $log);

					$data_arr = array('success' => TRUE, 'message' => $this->lang->line('manage_department_successmsg'), 'formaction' => 'reset');

				} else {
					$data_arr = array('success' => FALSE, 'message' => $this->lang->line('manage_department_errormsg'));
				}
			}
			else
			{
				$data_arr = array('success' => FALSE, 'message' => validation_errors());
			}
		} catch(Exception $e) {
			$data_arr = array("success" => FALSE, "message" => $e->getMessage());
		}

		die(json_encode($data_arr));
	}

	public function addNewQuick()
    {
    	General::validatePermission('add');

		if(isset($_POST['department_name']) AND $_POST['department_name'] != '') 
		{
			self::save();
		} 
		else 
		{
			$this->data['form_title'] = $this->lang->line('manage_department_addnew');
        	$this->load->view('quick_forms/add_department',$this->data);
		}
    }

}
