<?php

defined('BASEPATH') OR exit('No direct script access allowed');

require APPPATH.'controllers/General.php'; 


class Manage_designation extends General {

	private $editPermission;

	function __construct() {
		parent::__construct();	

		if(General::isLoggedIN() === FALSE OR General::validateToken() === FALSE){
            redirect(base_url());    
        }

        General::ModAccess(__CLASS__);
        self::getPermissionAccess();
        
        $this->load->model('designation_model');
	}

	private function getPermissionAccess()
	{
		$this->data['addPermission'] = General::validatePermission('add', FALSE);
        $this->data['deletePermission'] = General::validatePermission('delete', FALSE);
        $this->data['deactivatePermission'] = General::validatePermission('deactivate', FALSE);
        $this->data['activatePermission'] = General::validatePermission('activate', FALSE);
        $this->editPermission = General::validatePermission('edit', FALSE);
        $this->data['editPermission'] = $this->editPermission;
	}

	public function index()
	{
		$this->data['page_title'] = $this->lang->line('manage_designation_title');
		$this->data['page_sub_title'] = $this->lang->line('manage_designation_subtitle');
		$this->data['css'] = $this->assets_resources->getAssets('manage_designation_asset')['index']['css'];
		$this->data['js'] = $this->assets_resources->getAssets('manage_designation_asset')['index']['js'];
		$this->data['jsextend'] = $this->assets_resources->getAssets('manage_designation_asset')['index']['jsextend'];
		$this->data['file'] = $this->assets_resources->getAssets('manage_designation_asset')['index']['file'];
		$this->data['dt'] = array('datatable' => TRUE, 'path' => strtolower(__CLASS__) . '/manage', 'sort' => array(1, 'asc'));
		$this->load->view("page", $this->data);
	}

	public function manage()
	{
		if (isset($_REQUEST['actionType']) && $_REQUEST['actionType'] == 'group_action') 
		{
			General::processAction('designation', 'designation_id');
			self::datatable();
		} else if(isset($_REQUEST['action']) && $_REQUEST['action'] == 'filter') {
			self::datatable($search = TRUE);
		} else {
			self::datatable();
		}
	}

	private function datatable($search = FALSE)
	{
		$result = $this->designation_model->getAllDesignation('', '', $search, TRUE);
		$iTotalRecords = count($result);
		$iDisplayLength = intval($_REQUEST['length']);
		$iDisplayLength = $iDisplayLength < 0 ? $iTotalRecords : $iDisplayLength; 
		$iDisplayStart = intval($_REQUEST['start']);
		$sEcho = intval($_REQUEST['draw']);

		$records = array();

		$end = $iDisplayStart + $iDisplayLength;
		$end = $end > $iTotalRecords ? $iTotalRecords : $end;

		$rsData = $this->designation_model->getAllDesignation($end, $iDisplayStart, $search, TRUE);

		if($rsData) {
			foreach ($rsData as $rsData) 
			{
				$hideEdit = ($this->editPermission) ? "hide" : "";

				$status = ($rsData->nStatus) ? "success" : "danger";
				$statusName = ($rsData->nStatus) ? "Active" : "InActive";
				$records["data"][] = array(
					'<label class="mt-checkbox mt-checkbox-single mt-checkbox-outline"><input name="id[]" type="checkbox" class="checkboxes" value="'.$rsData->designation_id.'"/><span></span></label>',
					$rsData->designation_name,
					$rsData->designation_desc,
					'<span class="label label-sm label-'.$status.'">'.$statusName.'</span>',
					'<a href="'.base_url(__CLASS__ . "/edit/" . $rsData->designation_id).'" class="btn btn-xs btn-outline grey-salsa"><i class="fa fa-edit"></i> Edit</a>'
				);
			}
		} else {
			$records["data"][] = array('','','','','');
		}

		$records["draw"] = $sEcho;
		$records["recordsTotal"] = $iTotalRecords;
		$records["recordsFiltered"] = $iTotalRecords;

		echo json_encode($records);
	}

	public function edit($id = '')
	{
		General::validatePermission(__FUNCTION__);

		if(isset($_POST['designation_name']) AND $_POST['designation_name'] != '') 
		{
			self::editsave();
		} 
		else 
		{
			$id = (int) $id;
			$this->data['page_title'] = $this->lang->line('manage_dept_edit');
			$this->data['page_sub_title'] = $this->lang->line('manage_department_subtitle');
			$this->data['css'] = $this->assets_resources->getAssets('manage_designation_asset')['add']['css'];
			$this->data['js'] = $this->assets_resources->getAssets('manage_designation_asset')['add']['js'];
			$this->data['jsextend'] = $this->assets_resources->getAssets('manage_designation_asset')['add']['jsextend'];
			$this->data['file'] = DIRECTORY_SEPARATOR . strtolower(__CLASS__) . DIRECTORY_SEPARATOR . __FUNCTION__ . '.php';
			$this->data['rsDetails'] = $this->designation_model->getDesignationDetails($id);
			$this->load->view("page", $this->data);
		}
	}

	public function editsave()
	{
		General::addJSONResponseHeader();

		try {
			$this->form_validation->set_rules("designation_name","Designation Name","trim|xss_clean|required|callback_validate_designation");
			$this->form_validation->set_rules("designation_desc","Description","trim|xss_clean|required");
			$this->form_validation->set_error_delimiters('', '');

			if($this->form_validation->run())
			{	
				$result = $this->designation_model->save('update');
				if($result) 
				{
					// Record activity log
					$log = 'New Designation: <strong>'. $this->input->post('designation_name') . '</strong> has been updated by ' .General::getFullName();
					General::log('MANAGE DESIGNATON', 'UPDATE', $log);

					$data_arr = array('success' => TRUE, 'message' => $this->lang->line('manage_designation_successmsg'));

				} else {
					$data_arr = array('success' => FALSE, 'message' => $this->lang->line('manage_designation_errormsg'));
				}
			}
			else
			{
				$data_arr = array('success' => FALSE, 'message' => validation_errors());
			}
		} catch(Exception $e) {
			$data_arr = array("success" => FALSE, "message" => $e->getMessage());
		}

		die(json_encode($data_arr));
	}

	public function add()
	{
	 	General::validatePermission(__FUNCTION__);

		if(isset($_POST['designation_name']) AND $_POST['designation_name'] != '') 
		{
			self::save();
		} 
		else 
		{
			$this->data['page_title'] = $this->lang->line('manage_designation_addnew');
			$this->data['page_sub_title'] = $this->lang->line('manage_designation_title');
			$this->data['css'] = $this->assets_resources->getAssets('manage_designation_asset')['add']['css'];
			$this->data['js'] = $this->assets_resources->getAssets('manage_designation_asset')['add']['js'];
			$this->data['jsextend'] = $this->assets_resources->getAssets('manage_designation_asset')['add']['jsextend'];
			$this->data['file'] = $this->assets_resources->getAssets('manage_designation_asset')['add']['file'];
			$this->load->view("page", $this->data);
		}
	}

	public function validate_designation()
	{
		$data = array(
				'edit'		=>	(isset($_POST['designation_id']) AND $_POST['designation_id'] != '') ? array('designation_id',$this->input->post('designation_id')) : FALSE,
				'table'		=>	'designation',
				'fields'	=>	array('designation_name' => $this->input->post('designation_name')),
		);
		if(General::validateFields($data) === TRUE) {
			$this->form_validation->set_message( __FUNCTION__ ,'Designation Already Exists.');
			return FALSE;
		} else {
			return TRUE;
		}
	}

	public function save()
	{
		General::addJSONResponseHeader();

		try {
			$this->form_validation->set_rules("designation_name","Designation Name","trim|xss_clean|required|callback_validate_designation");
			$this->form_validation->set_rules("designation_desc","Description","trim|xss_clean|required");
			$this->form_validation->set_error_delimiters('', '');

			if($this->form_validation->run())
			{	
				$result = $this->designation_model->save('insert');
				if($result) 
				{
					// Record activity log
					$log = 'New Designation: <strong>'. $this->input->post('designation_name') . '</strong> has been added by ' .General::getFullName();
					General::log('MANAGE DESIGNATON', 'INSERT', $log);

					$data_arr = array('success' => TRUE, 'message' => $this->lang->line('manage_designation_successmsg'), 'formaction' => 'reset');

				} else {
					$data_arr = array('success' => FALSE, 'message' => $this->lang->line('manage_designation_errormsg'));
				}
			}
			else
			{
				$data_arr = array('success' => FALSE, 'message' => validation_errors());
			}
		} catch(Exception $e) {
			$data_arr = array("success" => FALSE, "message" => $e->getMessage());
		}

		die(json_encode($data_arr));
	}

	public function addNewQuick()
    {
    	General::validatePermission('add');

		if(isset($_POST['designation_name']) AND $_POST['designation_name'] != '') 
		{
			self::save();
		} 
		else 
		{
			$this->data['form_title'] = $this->lang->line('manage_designation_addnew');
        	$this->load->view('quick_forms/add_designation',$this->data);
		}
    }
	
}
