<?php

defined('BASEPATH') OR exit('No direct script access allowed');

require APPPATH.'controllers/General.php'; 

class Manage_group extends General {

	function __construct() {
		parent::__construct();	

		if(General::isLoggedIN() === FALSE OR General::validateToken() === FALSE){
            redirect(base_url());    
        }

        General::ModAccess(__CLASS__);
        self::getPermissionAccess();

        $this->load->model('general_model');
        $this->load->model('group_model');

        $this->data['mainMenus'] = $this->general_model->getMainMenus();
	}

	private function getPermissionAccess()
	{
		$this->data['addPermission'] = General::validatePermission('add', FALSE);
        $this->data['deletePermission'] = General::validatePermission('delete', FALSE);
        $this->data['deactivatePermission'] = General::validatePermission('deactivate', FALSE);
        $this->data['activatePermission'] = General::validatePermission('activate', FALSE);
        $this->editPermission = General::validatePermission('edit', FALSE);
        $this->data['editPermission'] = $this->editPermission;
	}

	public function index()
	{
		$this->data['page_title'] = $this->lang->line('manage_group_title');
		$this->data['page_sub_title'] = $this->lang->line('manage_group_subtitle');
		$this->data['css'] = $this->assets_resources->getAssets('manage_group_asset')['index']['css'];
		$this->data['js'] = $this->assets_resources->getAssets('manage_group_asset')['index']['js'];
		$this->data['jsextend'] = $this->assets_resources->getAssets('manage_group_asset')['index']['jsextend'];
		$this->data['file'] = $this->assets_resources->getAssets('manage_group_asset')['index']['file'];
		$this->data['dt'] = array('datatable' => TRUE, 'path' => __CLASS__ . '/manage', 'sort' => array(1, 'asc'));
		$this->load->view("page", $this->data);
	}

	public function manage()
	{
		if (isset($_REQUEST['actionType']) && $_REQUEST['actionType'] == 'group_action') 
		{
			General::processAction('user_group', 'group_id');
			self::datatable();
		} else if(isset($_REQUEST['action']) && $_REQUEST['action'] == 'filter') {
			self::datatable($search = TRUE);
		} else {
			self::datatable();
		}
	}

	private function datatable($search = FALSE)
	{
		$result = $this->group_model->getGroups('', '', $search, TRUE);
		$iTotalRecords = count($result);
		$iDisplayLength = intval($_REQUEST['length']);
		$iDisplayLength = $iDisplayLength < 0 ? $iTotalRecords : $iDisplayLength; 
		$iDisplayStart = intval($_REQUEST['start']);
		$sEcho = intval($_REQUEST['draw']);

		$records = array();

		$end = $iDisplayStart + $iDisplayLength;
		$end = $end > $iTotalRecords ? $iTotalRecords : $end;

		$rsGroup = $this->group_model->getGroups($end, $iDisplayStart, $search, TRUE);

		if($rsGroup) {
			foreach ($rsGroup as $rsData) 
			{
				$status = ($rsData->nStatus) ? "success" : "danger";
				$statusName = ($rsData->nStatus) ? "Active" : "InActive";
				$records["data"][] = array(
					'<label class="mt-checkbox mt-checkbox-single mt-checkbox-outline"><input name="id[]" type="checkbox" class="checkboxes" value="'.$rsData->group_id.'"/><span></span></label>',
					$rsData->group_name,
					$rsData->group_desc,
					'<span class="label label-sm label-'.$status.'">'.$statusName.'</span>',
					'<a href="'.base_url(__CLASS__ . "/edit/" . $rsData->group_id).'" class="btn btn-xs btn-outline grey-salsa"><i class="fa fa-edit"></i> Edit</a>'
				);
			}
		} else {
			$records["data"][] = array('','','','','');
		}

		$records["draw"] = $sEcho;
		$records["recordsTotal"] = $iTotalRecords;
		$records["recordsFiltered"] = $iTotalRecords;

		echo json_encode($records);
	}

	public function add()
	{
		General::validatePermission(__FUNCTION__);

		if(isset($_POST['group_name']) AND $_POST['group_name'] != '') 
		{
			self::save();
		} 
		else 
		{
			$this->data['page_title'] = $this->lang->line('manage_group_addnew');
			$this->data['page_sub_title'] = $this->lang->line('manage_group_subtitle');
			$this->data['css'] = $this->assets_resources->getAssets('manage_group_asset')['add']['css'];
			$this->data['js'] = $this->assets_resources->getAssets('manage_group_asset')['add']['js'];
			$this->data['jsextend'] = $this->assets_resources->getAssets('manage_group_asset')['add']['jsextend'];
			$this->data['file'] = $this->assets_resources->getAssets('manage_group_asset')['add']['file'];
			$this->load->view("page", $this->data);
		}
	}

	public function edit($id = '')
	{
		General::validatePermission(__FUNCTION__);

		if(isset($_POST['group_name']) AND $_POST['group_name'] != '') 
		{
			self::editsave();
		} 
		else 
		{
			$id = (int) $id;
			$this->data['page_title'] = $this->lang->line('manage_group_edit');
			$this->data['page_sub_title'] = $this->lang->line('manage_group_subtitle');
			$this->data['css'] = $this->assets_resources->getAssets('manage_group_asset')['add']['css'];
			$this->data['js'] = $this->assets_resources->getAssets('manage_group_asset')['add']['js'];
			$this->data['jsextend'] = $this->assets_resources->getAssets('manage_group_asset')['add']['jsextend'];
			$this->data['file'] = DIRECTORY_SEPARATOR . strtolower(__CLASS__) . DIRECTORY_SEPARATOR . __FUNCTION__ . '.php';
			$this->data['groupDetails'] = $this->group_model->getGroupDetails($id);
			$this->load->view("page", $this->data);
		}
		
	}

	public function editsave()
	{
		General::addJSONResponseHeader();

		try {
			$this->form_validation->set_rules("group_name","Group Name","trim|xss_clean|required");
			$this->form_validation->set_rules("group_desc","Group Description","trim|xss_clean");
			$this->form_validation->set_rules("modules[]","Permissions","trim|xss_clean|required");
			$this->form_validation->set_error_delimiters('', '');

			if($this->form_validation->run())
			{	
				// Get post values from checked checkbox
				$modules = $this->input->post('modules');
				$newgroupaccess = array();
				$parentmenu = $this->general_model->getParentMenus(NULL, $modules);
				if($parentmenu) {
					foreach($parentmenu as $pmenu) {
						array_push($newgroupaccess, $pmenu->parent_id);
					}
				}

				// Merge modules and new group access array
				$newaccessgroup = array_merge($modules, $newgroupaccess);

				// Update group table
				$this->group_model->save('update', $newaccessgroup);

				$group_id = $this->input->post('group_id');

				// Get group id on permission and remove group id to reset 
				$group_permission = $this->group_model->getGroupIDs();
				foreach($group_permission as $gp) {
					$gpc = unserialize($gp->group_id);
					$to_remove = array($group_id);
					$newgroupid = array_diff($gpc, $to_remove);

					// Update group id
					$this->group_model->updateNewGP($newgroupid, $gp->permission_id);
				}

				// Get post values from checked checkbox
				$permissions = $this->input->post('permissions');
				foreach ( $permissions as $rspermissions) {
					
					$currGroup = $this->group_model->getGroupsPermissions($rspermissions);
					if ($currGroup) {
						foreach ($currGroup as $currRsGroup) {

							if($currRsGroup->group_id != '') {
								$unserialize_group = unserialize($currRsGroup->group_id) ;
							} else {
								$unserialize_group = array();
							}

							// Add new group ID in table
							array_push($unserialize_group, $group_id);

							// Update group_permission table and add group id 
							$this->group_model->updatepermission($unserialize_group, $rspermissions);

						}
					}

				}
				

				// Record activity log
				$log = 'Group <strong>'. $this->input->post('group_name') . '</strong> has been updated by ' .General::getFullName();
				General::log('MANAGE GROUP', 'UPDATE', $log);

				$data_arr = array('success' => TRUE, 'message' => $this->lang->line('manage_group_successmsg'));
			}
			else
			{
				$data_arr = array('success' => FALSE, 'message' => validation_errors());
			}
		} catch(Exception $e) {
			$data_arr = array("success" => FALSE, "message" => $e->getMessage());
		}

		die(json_encode($data_arr));
	}

	public function save()
	{
		General::addJSONResponseHeader();

		try {
			$this->form_validation->set_rules("group_name","Group Name","trim|xss_clean|required");
			$this->form_validation->set_rules("group_desc","Group Description","trim|xss_clean");
			$this->form_validation->set_rules("modules[]","Permissions","trim|xss_clean|required");
			$this->form_validation->set_error_delimiters('', '');
			if($this->form_validation->run())
			{	
				// Get post values from checked checkbox
				$modules = $this->input->post('modules');
				$newgroupaccess = array();
				$parentmenu = $this->general_model->getParentMenus(NULL, $modules);
				if($parentmenu) {
					foreach($parentmenu as $pmenu) {
						array_push($newgroupaccess, $pmenu->parent_id);
					}
				}

				// Merge modules and new group access array
				$newaccessgroup = array_merge($modules, $newgroupaccess);

				$result = $this->group_model->save('insert', $newaccessgroup);
				if($result) 
				{
					$group_id = $result;

					// Get post values from checked checkbox
					$permissions = $this->input->post('permissions');
					foreach ( $permissions as $rspermissions) {
						
						$currGroup = $this->group_model->getGroupsPermissions($rspermissions);
						if ($currGroup) {
							foreach ($currGroup as $currRsGroup) {

								if($currRsGroup->group_id != '') {
									$unserialize_group = unserialize($currRsGroup->group_id) ;
								} else {
									$unserialize_group = array();
								}

								// Add new group ID in table
								array_push($unserialize_group, $group_id);

								// Update group_permission table and add group id 
								$this->group_model->updatepermission($unserialize_group, $rspermissions);

							}
						}

					}

					// Record activity log
					$log = 'Group <strong>'. $this->input->post('group_name') . '</strong> has been added by ' .General::getFullName();
					General::log('MANAGE GROUP', 'INSERT', $log);

					$data_arr = array('success' => TRUE, 'message' => $this->lang->line('manage_group_successmsg'), 'formaction' => 'reset');

				} else {
					$data_arr = array('success' => FALSE, 'message' => $this->lang->line('manage_group_errormsg'));
				}

				
			}
			else
			{
				$data_arr = array('success' => FALSE, 'message' => validation_errors());
			}
		} catch(Exception $e) {
			$data_arr = array("success" => FALSE, "message" => $e->getMessage());
		}

		die(json_encode($data_arr));
	}


	
}
