<?php

defined('BASEPATH') OR exit('No direct script access allowed');

require APPPATH.'controllers/General.php'; 


class Manage_patients extends General {

	private $editPermission, $columns;

	function __construct() {
		parent::__construct();	

		if(General::isLoggedIN() === FALSE OR General::validateToken() === FALSE){
            redirect(base_url());    
        }

        General::ModAccess(__CLASS__);
        self::getPermissionAccess();
        
        $this->load->model('patients_model');


	}

	private function getPermissionAccess()
	{
		$this->data['addPermission'] = General::validatePermission('add', FALSE);
        $this->data['deletePermission'] = General::validatePermission('delete', FALSE);
        $this->data['deactivatePermission'] = General::validatePermission('deactivate', FALSE);
        $this->data['activatePermission'] = General::validatePermission('activate', FALSE);
        $this->data['addConsultNotes'] = General::validatePermission('add_consult_note', FALSE);
        $this->data['addRegistrationForm'] = General::validatePermission('add_registration_form', FALSE);
        $this->data['addMedicalDocs'] = General::validatePermission('add_medical_record', FALSE);
        $this->editPermission = General::validatePermission('edit', FALSE);
        $this->data['editPermission'] = $this->editPermission;
	}

	public function index()
	{
		$this->data['page_title'] = $this->lang->line('manage_patients_title');
		$this->data['page_sub_title'] = $this->lang->line('manage_patients_subtitle');
		$this->data['css'] = $this->assets_resources->getAssets('manage_patients_asset')['index']['css'];
		$this->data['js'] = $this->assets_resources->getAssets('manage_patients_asset')['index']['js'];
		$this->data['jsextend'] = $this->assets_resources->getAssets('manage_patients_asset')['index']['jsextend'];
		$this->data['file'] = $this->assets_resources->getAssets('manage_patients_asset')['index']['file'];
		$this->data['dt'] = array('datatable' => TRUE, 'path' => strtolower(__CLASS__) . '/manage', 'sort' => array(1, 'asc'));
		$this->session->set_userdata('parent_id', 'manage_patients_search');
		$this->load->view("page", $this->data);
	}

	public function manage()
	{
		
		if (isset($_REQUEST['actionType']) && $_REQUEST['actionType'] == 'group_action') 
		{
			General::processAction('patients', 'id');
			self::datatable();
		} else if(isset($_REQUEST['action']) && $_REQUEST['action'] == 'filter') {
			self::datatable($search = TRUE);
		} else {
			self::datatable();
		}
	}

	private function datatable($search = FALSE)
	{
		

		$result = $this->patients_model->getAll('', '', $search, TRUE);
		$iTotalRecords = count($result);
		$iDisplayLength = intval($_REQUEST['length']);
		$iDisplayLength = $iDisplayLength < 0 ? $iTotalRecords : $iDisplayLength; 
		$iDisplayStart = intval($_REQUEST['start']);
		$sEcho = intval($_REQUEST['draw']);

		$records = array();
		$dtt = array();

		$end = $iDisplayStart + $iDisplayLength;
		$end = $end > $iTotalRecords ? $iTotalRecords : $end;

		$rsData = $this->patients_model->getAll($end, $iDisplayStart, $search, TRUE);

		if($rsData) {
			foreach ($rsData as $rsData) 
			{
				$hideEdit = ($this->editPermission) ? "hide" : "";

				$status = ($rsData->nStatus) ? "success" : "danger";
				$statusName = ($rsData->nStatus) ? "Active" : "InActive";
				$dtt[] = array(
					'<label class="mt-checkbox mt-checkbox-single mt-checkbox-outline"><input name="id[]" type="checkbox" class="checkboxes" value="'.$rsData->id.'"/><span></span></label>',
					'<a href="'.base_url(__CLASS__ . "/view/" . $rsData->id).'">'.ucwords($rsData->firstname.' '.$rsData->middlename.' '.$rsData->lastname).'</a>',
					(!empty($rsData->date_created)) ? date('d/M/Y', strtotime($rsData->date_created)) : '',
					ucwords($rsData->home_address.' '.$rsData->city.' '.$rsData->province.' '.$rsData->postal_code),
					ucwords($rsData->phone_number),
					'<a href="'.base_url(__CLASS__ . "/edit/" . $rsData->id).'" class="btn btn-xs btn-outline grey-salsa"><i class="fa fa-edit"></i> Edit</a>'
				);
			}
		} else {
			$dtt[] = array('','','','','','','');
		}

		$records["draw"] = $sEcho;
		$records["recordsTotal"] = $iTotalRecords;
		$records["recordsFiltered"] = $iTotalRecords;
		$records["data"] = $dtt;

		echo json_encode($records);
	}

	public function loadMedicalDocs($patient_id)
	{

		if($patient_id > 0) {
			$this->data['results'] = $this->patients_model->loadMedicalDocs($patient_id);
			$this->data['patient_id'] = $patient_id;
			$this->load->view('manage_patients/loadMedicalDocs', $this->data);
		} else {
			die('Invalid CMID');
		}
	}

	public function view($id, $view_medical_document = '')
	{
		$id = (int) $id;
		
		if($id > 0) {
			$this->data['page_title'] = 'View Patient Details';
			$this->data['page_sub_title'] = 'Manage Patients';
			$this->data['css'] = $this->assets_resources->getAssets('manage_patients_asset')['view']['css'];

			$this->data['js'] = $this->assets_resources->getAssets('manage_patients_asset')['view']['js'];
			$this->data['jsextend'] = $this->assets_resources->getAssets('manage_patients_asset')['view']['jsextend'];
			$this->data['file'] = DIRECTORY_SEPARATOR . strtolower(__CLASS__) . DIRECTORY_SEPARATOR . __FUNCTION__ . '.php';
			$this->data['rsDetails'] = $this->patients_model->getDetails($id);
			$this->data['view_medical_document'] = $view_medical_document;
			$this->session->set_userdata('parent_id', 'manage_patients_search');
			$this->data['last_gestational'] = $this->patients_model->getLastGestational($id);
			// $this->session->set_userdata('pageclosed', TRUE);
			$this->load->view("page", $this->data);
		} else {
			die('Invalid ID');
		}
	}

	public function edit($id = '')
	{
		General::validatePermission(__FUNCTION__);

		if(isset($_POST['firstname']) AND $_POST['firstname'] != '') 
		{
			self::editsave();
		} 
		else 
		{
			$id = (int) $id;
			$this->data['page_title'] = 'Edit Patient';
			$this->data['page_sub_title'] = 'Manage Patients';
			$this->data['css'] = $this->assets_resources->getAssets('manage_patients_asset')['add']['css'];
			$this->data['js'] = $this->assets_resources->getAssets('manage_patients_asset')['add']['js'];
			$this->data['jsextend'] = $this->assets_resources->getAssets('manage_patients_asset')['add']['jsextend'];
			$this->data['file'] = DIRECTORY_SEPARATOR . strtolower(__CLASS__) . DIRECTORY_SEPARATOR . __FUNCTION__ . '.php';
			$this->data['rsDetails'] = $this->patients_model->getDetails($id);
			$this->session->set_userdata('parent_id', 'manage_patients_search');
			$this->load->view("page", $this->data);
		}
	}

	public function delete_record($id)
	{
		$del = $this->patients_model->delete_record($id);
		$result = ($del) ? array('success'=>TRUE) : array('success'=>FALSE);
		return die(json_encode($result));
	}

	public function record_pdf($id)
	{
		$rsDetails = $this->patients_model->getRecord($id);
		$this->data['rsDetails'] = $rsDetails;
		$html = $this->load->view("pdf/medical_document", $this->data, TRUE);
		$filename = ucfirst($rsDetails->lastname)."_".ucfirst($rsDetails->firstname)."_".date('Ymd')."_MedDoc";
		$papersize = "A4";
		$orientation = 'portrait'; // landscape or portrait
		pdf_create($html, $filename, TRUE, $papersize, $orientation);
		return $filename;

	}

	public function edit_record()
	{
		General::addJSONResponseHeader();

		try {
			$this->form_validation->set_rules("weight","Weight","trim|xss_clean");
			$this->form_validation->set_rules("bp","Blood Pressure","trim|xss_clean|required");
			$this->form_validation->set_rules("temp","Body Temperature","trim|xss_clean");
			$this->form_validation->set_rules("age_gestation","Age of gestation","trim|xss_clean");
			$this->form_validation->set_rules("fetal_heart_beat","Fetal heart beat/presentation/est. fetal wt","trim|xss_clean");
			$this->form_validation->set_rules("chief_complain","Chief complain","trim|xss_clean");
			$this->form_validation->set_rules("internal_exam","Internal Exam","trim|xss_clean");
			$this->form_validation->set_rules("lab_results","Lab results","trim|xss_clean");
			$this->form_validation->set_rules("comments","Comments","trim|xss_clean");
			$this->form_validation->set_error_delimiters('', '');

			if($this->form_validation->run())
			{	
				$upload_data = array();
				// $fileErr = FALSE;
				// $uploadErr = "";

				// if (!empty($_FILES['file_docs']['name'])) {
				// 	$filename = 'files-'.strtotime(date("His"));

				// 	$config = array(
				// 		'allowed_types'		=>		'*',
				// 		'upload_path'		=>		realpath('public/files/patients'),
				// 		'max_size'			=>		20000,
				// 		'file_name'			=>		$filename,
				// 		'overwrite'			=>		TRUE
				// 	);

				// 	$this->load->library('upload', $config);

				// 	if ($this->upload->do_upload('file_docs'))
			 //        {
			 //            $upload_data = $this->upload->data();

			 //            if(!empty($this->input->post('old_file'))) {
			 //            	$oldFile = FCPATH . 'public/files/patients/' . $this->input->post('old_file');
			 //            	unlink($oldFile);
			 //            }
			            
			 //        } else {
			 //        	$fileErr = TRUE;
			 //        	$uploadErr = $this->upload->display_errors();
			 //        }
				// }

				// if($fileErr) {
				// 	$data_arr = array('success' => FALSE, 'message' => $uploadErr);
				// } else {
				// 	$result = $this->patients_model->save_record('update', $upload_data);
				// 	if($result) 
				// 	{
				// 		// Record activity log
				// 		$log = 'New Patient Medical Record has been updated by ' .General::getFullName();
				// 		General::log('PATIENT MEDICAL RECORD', 'UPDATE', $log);

				// 		$data_arr = array('success' => TRUE, 'message' => 'Patient medical record has been saved.', 'formaction' => 'patient_medical_edit');

				// 	} else {
				// 		$data_arr = array('success' => FALSE, 'message' => 'Their\'s an error in saving patient.');
				// 	}
				// }


				$result = $this->patients_model->save_record('update', $upload_data);
				if($result) 
				{
					// Record activity log
					$log = 'New Patient record has been updated by ' .General::getFullName();
					General::log('PATIENT RECORD', 'UPDATE', $log);

					$data_arr = array('success' => TRUE, 'message' => 'Patient record has been saved.', 'formaction' => 'patient_medical_edit');

				} else {
					$data_arr = array('success' => FALSE, 'message' => 'Their\'s an error in saving patient.');
				}
				
			}
			else
			{
				$data_arr = array('success' => FALSE, 'message' => validation_errors());
			}
		} catch(Exception $e) {
			$data_arr = array("success" => FALSE, "message" => $e->getMessage());
		}

		die(json_encode($data_arr));
	}

	public function save_record()
	{
		General::addJSONResponseHeader();

		try {
			$this->form_validation->set_rules("weight","Weight","trim|xss_clean");
			$this->form_validation->set_rules("bp","Blood Pressure","trim|xss_clean|required");
			$this->form_validation->set_rules("temp","Body Temperature","trim|xss_clean");
			$this->form_validation->set_rules("age_gestation","Age of gestation","trim|xss_clean");
			$this->form_validation->set_rules("fetal_heart_beat","Fetal heart beat/presentation/est. fetal wt","trim|xss_clean");
			$this->form_validation->set_rules("chief_complain","Chief complain","trim|xss_clean");
			$this->form_validation->set_rules("internal_exam","Internal Exam","trim|xss_clean");
			$this->form_validation->set_rules("lab_results","Lab results","trim|xss_clean");
			$this->form_validation->set_rules("comments","Comments","trim|xss_clean");
			$this->form_validation->set_error_delimiters('', '');

			if($this->form_validation->run())
			{	

				$upload_data = array();
				// $fileErr = FALSE;
				// $uploadErr = "";

				// if (!empty($_FILES['file_docs']['name'])) {
				// 	$filename = 'files-'.strtotime(date("His"));

				// 	$config = array(
				// 		'allowed_types'		=>		'*',
				// 		'upload_path'		=>		realpath('public/files/patients'),
				// 		'max_size'			=>		20000,
				// 		'file_name'			=>		$filename,
				// 		'overwrite'			=>		TRUE
				// 	);

				// 	$this->load->library('upload', $config);

				// 	if ($this->upload->do_upload('file_docs'))
			 //        {
			 //            $upload_data = $this->upload->data();
			 //        } else {
			 //        	$fileErr = TRUE;
			 //        	$uploadErr = $this->upload->display_errors();
			 //        }
				// }

				// if($fileErr) {
				// 	$data_arr = array('success' => FALSE, 'message' => $uploadErr);
				// } else {
				// 	$result = $this->patients_model->save_record('insert', $upload_data);
				// 	if($result) 
				// 	{
				// 		// Record activity log
				// 		$log = "New Patient Medical Record has been saved by ".General::getFullName()."<br />Patient id: " . $this->input->post('patient_id');
				// 		General::log('PATIENT MEDICAL RECORD', 'INSERT', $log);

				// 		$data_arr = array('success' => TRUE, 'message' => 'Patient medical record has been saved.', 'formaction' => 'patient_medical');

				// 	} else {
				// 		$data_arr = array('success' => FALSE, 'message' => 'Their\'s an error in saving patient.');
				// 	}
				// }

				$result = $this->patients_model->save_record('insert', $upload_data);
				if($result) 
				{
					// Record activity log
					$log = "New Patient record has been saved by ".General::getFullName()."<br />Patient id: " . $this->input->post('patient_id');
					General::log('PATIENT RECORD', 'INSERT', $log);

					$data_arr = array('success' => TRUE, 'message' => 'Patient record has been saved.', 'formaction' => 'patient_medical');

				} else {
					$data_arr = array('success' => FALSE, 'message' => 'Their\'s an error in saving patient.');
				}

				
			}
			else
			{
				$data_arr = array('success' => FALSE, 'message' => validation_errors());
			}
		} catch(Exception $e) {
			$data_arr = array("success" => FALSE, "message" => $e->getMessage());
		}

		die(json_encode($data_arr));
	}

	public function editsave()
	{
		General::addJSONResponseHeader();

		try {
			$this->form_validation->set_rules("firstname","First Name","trim|xss_clean|required");
			$this->form_validation->set_rules("lastname","Last Name","trim|xss_clean|required");
			$this->form_validation->set_rules("middlename","Middle Name","trim|xss_clean");
			$this->form_validation->set_rules("email_address","Email Address","trim|xss_clean");
			$this->form_validation->set_rules("dob","Date of Birth","trim|xss_clean|required");
			$this->form_validation->set_rules("phone_number","Phone Number","trim|xss_clean");
			$this->form_validation->set_rules("cmid","CMID","trim|xss_clean");
			$this->form_validation->set_rules("home_address","Home Address","trim|xss_clean|required");
			$this->form_validation->set_rules("health_card_number","Health Card Number","trim|xss_clean");
			$this->form_validation->set_rules("estimated_delivery_date","Estimated Date of Delivery","trim|xss_clean");
			$this->form_validation->set_rules("gestation_week_day","Gestation Today","trim|xss_clean");
			$this->form_validation->set_rules("notes","Notes","trim|xss_clean");
			$this->form_validation->set_error_delimiters('', '');
			if($this->form_validation->run())
			{	
				$result = $this->patients_model->save('update');
				if($result) 
				{
					// Record activity log
					$log = 'New Patient: <strong>'. $this->input->post('firstname').' '.$this->input->post('lastname') . '</strong> has been updated by ' .General::getFullName();
					General::log('MANAGE DESIGNATON', 'UPDATE', $log);

					$data_arr = array('success' => TRUE, 'message' => 'Patient has been updated.', 'formaction' => 'view_patient', 'last_id' => $this->input->post('id'));

				} else {
					$data_arr = array('success' => FALSE, 'message' => 'Their\'s an error in saving patient.');
				}
			}
			else
			{
				$data_arr = array('success' => FALSE, 'message' => validation_errors());
			}
		} catch(Exception $e) {
			$data_arr = array("success" => FALSE, "message" => $e->getMessage());
		}

		die(json_encode($data_arr));
	}

	public function add()
	{
	 	General::validatePermission(__FUNCTION__);

		if(isset($_POST['firstname']) AND $_POST['firstname'] != '') 
		{
			self::save();
		} 
		else 
		{
			$this->data['page_title'] = 'Add New Patient';
			$this->data['page_sub_title'] = 'Manage Patients';
			$this->data['css'] = $this->assets_resources->getAssets('manage_patients_asset')['add']['css'];
			$this->data['js'] = $this->assets_resources->getAssets('manage_patients_asset')['add']['js'];
			$this->data['jsextend'] = $this->assets_resources->getAssets('manage_patients_asset')['add']['jsextend'];
			$this->data['file'] = $this->assets_resources->getAssets('manage_patients_asset')['add']['file'];
			$this->session->set_userdata('parent_id', 'add_patients');
			$this->load->view("page", $this->data);
		}
	}

	public function validate_designation()
	{
		$data = array(
				'edit'		=>	(isset($_POST['designation_id']) AND $_POST['designation_id'] != '') ? array('designation_id',$this->input->post('designation_id')) : FALSE,
				'table'		=>	'designation',
				'fields'	=>	array('designation_name' => $this->input->post('designation_name')),
		);
		if(General::validateFields($data) === TRUE) {
			$this->form_validation->set_message( __FUNCTION__ ,'Designation Already Exists.');
			return FALSE;
		} else {
			return TRUE;
		}
	}

	public function save()
	{
		General::addJSONResponseHeader();
		$cmid = General::getCMID() + 1;

		try {
			$this->form_validation->set_rules("firstname","First Name","trim|xss_clean|required");
			$this->form_validation->set_rules("lastname","Last Name","trim|xss_clean|required");
			$this->form_validation->set_rules("middlename","Middle Name","trim|xss_clean");
			$this->form_validation->set_rules("email_address","Email Address","trim|xss_clean");
			$this->form_validation->set_rules("dob","Date of Birth","trim|xss_clean|required");
			$this->form_validation->set_rules("gender","Gender","trim|xss_clean|required");
			$this->form_validation->set_rules("phone_number","Phone Number","trim|xss_clean");
			$this->form_validation->set_rules("cmid","CMID","trim|xss_clean");
			$this->form_validation->set_rules("home_address","Home Address","trim|xss_clean|required");
			$this->form_validation->set_rules("health_card_number","Health Card Number","trim|xss_clean");
			$this->form_validation->set_rules("notes","Notes","trim|xss_clean");
			// $this->form_validation->set_rules("estimated_delivery_date","Estimated Date of Delivery","trim|xss_clean");
			// $this->form_validation->set_rules("gestation_week_day","Gestation Today","trim|xss_clean");
			$this->form_validation->set_error_delimiters('', '');

			if($this->form_validation->run())
			{	
				$result = $this->patients_model->save('insert', $cmid);
				if($result) 
				{

					// save gestational age
					// if(!empty($this->input->post('estimated_delivery_date')) AND !empty($this->input->post('gestation_week_day'))) {
					// 	$this->patients_model->saveGestationalAge('insert', $result);
					// }

					// insert patient record
					if(!empty($_REQUEST['weight']) OR !empty($_REQUEST['bp']) OR !empty($_REQUEST['temp']) OR !empty($_REQUEST['age_gestation']) 
						OR !empty($_REQUEST['chief_complain']) OR !empty($_REQUEST['fetal_heart_beat']) OR !empty($_REQUEST['internal_exam']) OR !empty($_REQUEST['lab_results']) OR !empty($_REQUEST['comments'])) {
						$this->patients_model->savePatientRecord('insert', $result);
					}

					// save notes
					if(!empty($this->input->post('notes'))) {
						$this->patients_model->saveNotes('insert', $result);
					}

					// Record activity log
					$log = 'New Patient: <strong>'. $this->input->post('firstname').' '.$this->input->post('lastname') . '</strong> has been added by ' .General::getFullName();
					General::log('MANAGE PATIENTS', 'INSERT', $log);

					$this->patients_model->updateCMID($cmid);

					$data_arr = array('success' => TRUE, 'message' => 'Patients has been saved', 'formaction' => 'view_patient', 'last_id' => $result);

				} else {
					$data_arr = array('success' => FALSE, 'message' => 'Their\'s an error in saving patient.');
				}
			}
			else
			{
				$data_arr = array('success' => FALSE, 'message' => validation_errors());
			}
		} catch(Exception $e) {
			$data_arr = array("success" => FALSE, "message" => $e->getMessage());
		}

		die(json_encode($data_arr));
	}

	public function addNewQuick()
    {
    	General::validatePermission('add');

		if(isset($_POST['designation_name']) AND $_POST['designation_name'] != '') 
		{
			self::save();
		} 
		else 
		{
			$this->data['form_title'] = $this->lang->line('manage_designation_addnew');
        	$this->load->view('quick_forms/add_designation',$this->data);
		}
    }

    public function getRecord($id)
    {
    	$result = $this->patients_model->getMedicalRecord($id);
    	die(json_encode($result));
    }

    public function quick_add()
    {
    	General::addJSONResponseHeader();
    	$cmid = General::getCMID() + 1;

		try {
			$this->form_validation->set_rules("firstname","First Name","trim|xss_clean|required");
			$this->form_validation->set_rules("lastname","Last Name","trim|xss_clean|required");
			$this->form_validation->set_rules("phone_number","LMP","trim|xss_clean|required");
			$this->form_validation->set_rules("home_address","Home Address","trim|xss_clean|required");
			$this->form_validation->set_error_delimiters('', '');

			if($this->form_validation->run())
			{	
				$result = $this->patients_model->save('insert', $cmid);
				if($result) 
				{
					// Record activity log
					$log = 'New Patient: <strong>'. $this->input->post('firstname').' '.$this->input->post('lastname') . '</strong> has been added by ' .General::getFullName();
					General::log('MANAGE PATIENTS', 'INSERT', $log);

					$this->patients_model->updateCMID($cmid);

					$data_arr = array('success' => TRUE, 'message' => 'Patients has been saved', 'patient_id' => $result);

				} else {
					$data_arr = array('success' => FALSE, 'message' => 'Their\'s an error in saving patient.');
				}
			}
			else
			{
				$data_arr = array('success' => FALSE, 'message' => validation_errors());
			}
		} catch(Exception $e) {
			$data_arr = array("success" => FALSE, "message" => $e->getMessage());
		}

		die(json_encode($data_arr));
    }

    public function getPatientAppointments()
    {
    	$dateRange = $this->input->post('date_range');
		$dateStart = date("Y-m-d", strtotime(explode("-", $dateRange)[0]));
		$dateend = date("Y-m-d", strtotime(explode("-", $dateRange)[1]));
		$patient_id = $this->input->post('patient_id');

		try {
			$this->form_validation->set_rules("date_range","Appointment Date","trim|xss_clean|required");
			$this->form_validation->set_error_delimiters('', '');

			if($this->form_validation->run())
			{	
				$result = $this->patients_model->getPatientAppointments($dateStart, $dateend, $patient_id);
				if($result) 
				{
					$this->data['patients_appointments'] = $result;
					$this->load->view('manage_patients/patients_appointments', $this->data);
				} else {
					echo 'No appointments found.';
				}
			}
			else
			{
				echo 'validation_errors()';
			}
		} catch(Exception $e) {
			echo 'Appointment Date';
		}
    }
	
}
