<?php

defined('BASEPATH') OR exit('No direct script access allowed');

require APPPATH.'controllers/General.php'; 

class Manage_users extends General {

	private $editPermission;

	function __construct() {
		parent::__construct();	

		if(General::isLoggedIN() === FALSE OR General::validateToken() === FALSE){
            redirect(base_url());    
        }

        General::ModAccess(__CLASS__);
        self::getPermissionAccess();
        
        $this->load->model('group_model');
        $this->load->model('user_model');
        $this->load->model('department_model');
        $this->load->model('designation_model');
	}

	private function getPermissionAccess()
	{
		$this->data['addPermission'] = General::validatePermission('add', FALSE);
        $this->data['deletePermission'] = General::validatePermission('delete', FALSE);
        $this->data['deactivatePermission'] = General::validatePermission('deactivate', FALSE);
        $this->data['activatePermission'] = General::validatePermission('activate', FALSE);
        $this->editPermission = General::validatePermission('edit', FALSE);
        $this->data['editPermission'] = $this->editPermission;
	}

	public function index()
	{
		$this->data['page_title'] = $this->lang->line('manage_users_title');
		$this->data['page_sub_title'] = $this->lang->line('manage_users_subtitle');
		$this->data['css'] = $this->assets_resources->getAssets('manage_users_asset')['index']['css'];
		$this->data['js'] = $this->assets_resources->getAssets('manage_users_asset')['index']['js'];
		$this->data['jsextend'] = $this->assets_resources->getAssets('manage_users_asset')['index']['jsextend'];
		$this->data['file'] = $this->assets_resources->getAssets('manage_users_asset')['index']['file'];
		$this->data['dt'] = array('datatable' => TRUE, 'path' => __CLASS__ . '/manage', 'sort' => array(1, 'asc'));
		$this->data['groupList'] = $this->group_model->getGroups();
		$this->load->view("page", $this->data);
	}

	public function manage()
	{
		if (isset($_REQUEST['actionType']) && $_REQUEST['actionType'] == 'group_action') 
		{
			General::processAction('users', 'user_id');
			self::datatable();
		} else if(isset($_REQUEST['action']) && $_REQUEST['action'] == 'filter') {
			self::datatable($search = TRUE);
		} else {
			self::datatable();
		}
	}

	private function datatable($search = FALSE)
	{
		$result = $this->user_model->getUsers('', '', $search, TRUE);
		$iTotalRecords = count($result);
		$iDisplayLength = intval($_REQUEST['length']);
		$iDisplayLength = $iDisplayLength < 0 ? $iTotalRecords : $iDisplayLength; 
		$iDisplayStart = intval($_REQUEST['start']);
		$sEcho = intval($_REQUEST['draw']);

		$records = array();

		$end = $iDisplayStart + $iDisplayLength;
		$end = $end > $iTotalRecords ? $iTotalRecords : $end;

		$rsUsers = $this->user_model->getUsers($end, $iDisplayStart, $search, TRUE);

		if($rsUsers) {
			foreach ($rsUsers as $rsData) 
			{
				$hideEdit = ($this->editPermission) ? "hide" : "";

				$status = ($rsData->nStatus) ? "success" : "danger";
				$statusName = ($rsData->nStatus) ? "Active" : "InActive";
				$records["data"][] = array(
					'<label class="mt-checkbox mt-checkbox-single mt-checkbox-outline"><input name="id[]" type="checkbox" class="checkboxes" value="'.$rsData->user_id.'"/><span></span></label>',
					'<a href=\''.base_url().strtolower(__CLASS__).'/view/'.$rsData->user_id.'\'>'.$rsData->name.'</a>',
					$rsData->email,
					$rsData->phone,
					$rsData->group_name,
					'<span class="label label-sm label-'.$status.'">'.$statusName.'</span>',
					'<a href="'.base_url(__CLASS__ . "/edit/" . $rsData->user_id).'" class="btn btn-xs btn-outline grey-salsa"><i class="fa fa-edit"></i> Edit</a>'
				);
			}
		} else {
			$records["data"][] = array('','','','','','','');
		}

		$records["draw"] = $sEcho;
		$records["recordsTotal"] = $iTotalRecords;
		$records["recordsFiltered"] = $iTotalRecords;

		echo json_encode($records);
	}

	public function edit($id = '')
	{
		General::validatePermission(__FUNCTION__);

		if(isset($_POST['firstname']) AND $_POST['firstname'] != '') 
		{
			self::editsave();
		} 
		else 
		{
			$id = (int) $id;
			$this->data['page_title'] = $this->lang->line('manage_users_edit');
			$this->data['page_sub_title'] = $this->lang->line('manage_users_subtitle');
			$this->data['css'] = $this->assets_resources->getAssets('manage_users_asset')['add']['css'];
			$this->data['js'] = $this->assets_resources->getAssets('manage_users_asset')['add']['js'];
			$this->data['jsextend'] = $this->assets_resources->getAssets('manage_users_asset')['add']['jsextend'];
			$this->data['file'] = DIRECTORY_SEPARATOR . strtolower(__CLASS__) . DIRECTORY_SEPARATOR . __FUNCTION__ . '.php';
			$this->data['groupList'] = $this->group_model->getGroups();
			$this->data['deptList'] = $this->department_model->getAllDepartment();
			$this->data['designationList'] = $this->designation_model->getAllDesignation();
			$this->data['userDetails'] = $this->user_model->getUserDetails(NULL, $id);
			$this->load->view("page", $this->data);
		}
	}

	public function editsave()
	{
		General::addJSONResponseHeader();

		try {
			$this->form_validation->set_rules("firstname","First Name","trim|xss_clean|required");
			$this->form_validation->set_rules("lastname","Last Name","trim|xss_clean|required");
			$this->form_validation->set_rules("group_id","User Group","trim|xss_clean|required");
			$this->form_validation->set_rules("department_id","Department","trim|xss_clean|required");
			$this->form_validation->set_rules("designation_id","Designation","trim|xss_clean|required");
			$this->form_validation->set_rules("email","Email Address","trim|xss_clean|required|callback_validate_email");
			$this->form_validation->set_rules("phone","Contact No.","trim|xss_clean");
			$this->form_validation->set_error_delimiters('', '');

			if($this->form_validation->run())
			{	
				$result = $this->user_model->save('update');
				if($result) 
				{
					// Record activity log
					$log = 'New User: <strong>'. $this->input->post('firstname'). ' ' . $this->input->post('lastname') . '</strong> has been updated by ' .General::getFullName();
					General::log('MANAGE USER', 'UPDATE', $log);

					$data_arr = array('success' => TRUE, 'message' => $this->lang->line('manage_users_successmsg'));

				} else {
					$data_arr = array('success' => FALSE, 'message' => $this->lang->line('manage_users_errormsg'));
				}

				
			}
			else
			{
				$data_arr = array('success' => FALSE, 'message' => validation_errors());
			}
		} catch(Exception $e) {
			$data_arr = array("success" => FALSE, "message" => $e->getMessage());
		}

		die(json_encode($data_arr));
	}

	public function add()
	{
	 	General::validatePermission(__FUNCTION__);

		if(isset($_POST['firstname']) AND $_POST['firstname'] != '') 
		{
			self::save();
		} 
		else 
		{
			$this->data['page_title'] = $this->lang->line('manage_users_title');
			$this->data['page_sub_title'] = $this->lang->line('manage_users_subtitle');
			$this->data['css'] = $this->assets_resources->getAssets('manage_users_asset')['add']['css'];
			$this->data['js'] = $this->assets_resources->getAssets('manage_users_asset')['add']['js'];
			$this->data['jsextend'] = $this->assets_resources->getAssets('manage_users_asset')['add']['jsextend'];
			$this->data['file'] = $this->assets_resources->getAssets('manage_users_asset')['add']['file'];
			$this->data['groupList'] = $this->group_model->getGroups();
			$this->data['deptList'] = $this->department_model->getAllDepartment();
			$this->data['designationList'] = $this->designation_model->getAllDesignation();
			$this->load->view("page", $this->data);
		}
	}

	public function validate_email()
	{
		$data = array(
				'edit'		=>	(isset($_POST['user_id']) AND $_POST['user_id'] != '') ? array('user_id',$this->input->post('user_id')) : FALSE,
				'table'		=>	'users',
				'fields'	=>	array('email' => $this->input->post('email')),
		);
		if(General::validateFields($data) === TRUE) {
			$this->form_validation->set_message( __FUNCTION__ ,'Email Address Already Exists.');
			return FALSE;
		} else {
			return TRUE;
		}
	}

	public function save()
	{
		General::addJSONResponseHeader();

		try {
			$this->form_validation->set_rules("firstname","First Name","trim|xss_clean|required");
			$this->form_validation->set_rules("lastname","Last Name","trim|xss_clean|required");
			$this->form_validation->set_rules("group_id","User Group","trim|xss_clean|required");
			$this->form_validation->set_rules("department_id","Department","trim|xss_clean|required");
			$this->form_validation->set_rules("designation_id","Designation","trim|xss_clean|required");
			$this->form_validation->set_rules("email","Email Address","trim|xss_clean|required|callback_validate_email");
			$this->form_validation->set_rules("phone","Contact No.","trim|xss_clean");
			$this->form_validation->set_error_delimiters('', '');

			if($this->form_validation->run())
			{	

				$username = General::generateRandomString(8);
				$password = General::generateRandomString(12);
				$hashPassword = $this->password->hash($password);

				$result = $this->user_model->save('insert', $username, $hashPassword);
				if($result) 
				{

					$this->data['firstname'] = $this->input->post('firstname');
					$this->data['username'] = $username;
					$this->data['password'] = $password;
					$body_email = $this->load->view('email_template/new_user', $this->data, true);
					$this->email->set_mailtype("html");
					$this->email->from(General::EMAIL_FROM, General::EMAIL_FROM_NAME);
					$this->email->to($this->input->post('email'));
					$this->email->subject(General::SHORT_SYS_TITLE . ' - Account Credentials');
					$this->email->message($body_email);
					$send = $this->email->send();

					// Record activity log
					$log = 'New User: <strong>' . $this->input->post('firstname'). ' ' . $this->input->post('lastname') . '</strong> has been added by ' .General::getFullName();
					General::log('MANAGE USER', 'INSERT', $log);

					$data_arr = array('success' => TRUE, 'message' => $this->lang->line('manage_users_successmsg'), 'formaction' => 'reset');

				} else {
					$data_arr = array('success' => FALSE, 'message' => $this->lang->line('manage_users_errormsg'));
				}

				
			}
			else
			{
				$data_arr = array('success' => FALSE, 'message' => validation_errors());
			}
		} catch(Exception $e) {
			$data_arr = array("success" => FALSE, "message" => $e->getMessage());
		}

		die(json_encode($data_arr));
	}

	public function view($id)
	{
		$this->data['page_title'] = $this->lang->line('manage_users_info');
		$this->data['page_sub_title'] = $this->lang->line('manage_users_title');
		$this->data['css'] = $this->assets_resources->getAssets('manage_users_asset')['view']['css'];
		$this->data['js'] = $this->assets_resources->getAssets('manage_users_asset')['view']['js'];
		$this->data['jsextend'] = $this->assets_resources->getAssets('manage_users_asset')['view']['jsextend'];
		$this->data['file'] = $this->assets_resources->getAssets('manage_users_asset')['view']['file'];
		$this->data['user_details'] = $this->user_model->getUserDetails(NULL, $id);
		$this->data['user_log'] = $this->user_model->getUserLog($id);
		$this->data['table'] = 'users';
		$this->data['field_id'] = 'user_id';
		$this->data['field_id_value'] = $id;
		$this->load->view("page", $this->data);
	}


	
}
