<?php 

if ( ! defined('BASEPATH')) exit('No direct script access allowed');

require APPPATH.'controllers/General.php'; 

class Patients_appointment_report extends General{

	function __construct() {
		parent::__construct();	

		if(General::isLoggedIN() === FALSE OR General::validateToken() === FALSE){
            redirect(base_url());    
        }

        General::ModAccess(__CLASS__);
        self::getPermissionAccess();
        
        $this->load->model('patients_model');
	}

	private function getPermissionAccess()
	{
		$this->data['printPermission'] = General::validatePermission('print_report', FALSE);
        $this->data['excelPermission'] = General::validatePermission('export_excel', FALSE);
        $this->data['pdfPermission'] = General::validatePermission('export_PDF', FALSE);
        $this->data['csvPermission'] = General::validatePermission('export_CSV', FALSE);
	}

	public function index()
	{
		$this->data['page_title'] = 'Reports';
		$this->data['page_sub_title'] = 'Patients Appointment Report';
		$this->data['css'] = $this->assets_resources->getAssets('patients_report_asset')['index']['css'];
		$this->data['js'] = $this->assets_resources->getAssets('patients_report_asset')['index']['js'];
		$this->data['jsextend'] = $this->assets_resources->getAssets('patients_report_asset')['index']['jsextend'];
		$this->data['file'] = '/patients_appointment_report/index.php';
		$this->load->view("page", $this->data);
	}


	public function print_report()
	{

		$printOption = $this->input->post("printOption");

		if($printOption == "web_print")
		{
			$this->result_print();

			$log = 'Patients appointment report has been printed by ' .General::getFullName();
			General::log('PATIENTS APPOINTMENT REPORT', 'PRINT', $log);

		}
		else if($printOption == "export_excel")
		{
			$this->result_excel();

			$log = 'Patients appointment report has been exported to excel by ' .General::getFullName();
			General::log('PATIENTS APPOINTMENT REPORT', 'EXPORT EXCEL', $log);

		}
		else if($this->input->post('printOption') == "export_pdf")
		{
			$this->result_pdf();

			$log = 'Patients appointment report has been exported to PDF by ' .General::getFullName();
			General::log('PATIENTS APPOINTMENT REPORT', 'EXPORT PDF', $log);

		}
		else if($this->input->post('printOption') == "export_csv")
		{
			$this->ExportCSV();

			$log = 'Patients appointment report has been exported to CSV by ' .General::getFullName();
			General::log('PATIENTS APPOINTMENT REPORT', 'EXPORT CSV', $log);

		}

		
	}

	public function result_pdf()
	{
		$dateRange = $this->input->post('date_range');
		$data = array('dateRange'=>(trim($dateRange) == FALSE) ? "" : $dateRange);
		$this->data['page_title'] = "Patients Appointment Report";
		$this->data['print'] = FALSE;
		$this->data['rsDetails'] = $this->patients_model->getPatientsAppReport();
		$this->data['dateRange'] = $dateRange;
		$html = $this->load->view('patients_appointment_report/print', $this->data, true);
		$filename = "patients_apppointment_report_".date("YmdHis");
		$papersize = "A4";

		$orientation = $this->input->post('pdfOrientation');
        pdf_create($html, $filename, TRUE, $papersize, $orientation);
	}

	public function result_print()
	{
		$dateRange = $this->input->post('date_range');
		$data = array('dateRange'=>(trim($dateRange) == FALSE) ? "" : $dateRange);
		$this->data['page_title'] = "Patients Appointment Report";
		$this->data['print'] = TRUE;
		$this->data['dateRange'] = $dateRange;
		$this->data['rsDetails'] = $this->patients_model->getPatientsAppReport();
		$this->load->view('patients_appointment_report/print', $this->data);
	}

	public function ExportCSV()
	{
        $this->load->dbutil();
        $this->load->helper('file');
        $this->load->helper('download');
        $delimiter = ",";
        $newline = "\r\n";
        $filename = "patients_apppointment_report_".date("YmdHis").'.csv';

		$result = $this->patients_model->getPatientsAppReport(TRUE);

        $data = $this->dbutil->csv_from_result($result, $delimiter, $newline);
        force_download($filename, $data);
	}


	public function result_excel()
	{
		$dateRange = $this->input->post('date_range');

		//load our new PHPExcel library
		$this->load->library('excel');
		//activate worksheet number 1
		$this->excel->setActiveSheetIndex(0);
		//name the worksheet
		$this->excel->getActiveSheet()->setTitle('Patients Appointment Report');
		$this->excel->getProperties()->setCreator("Company Name")
									 ->setLastModifiedBy('Name of Modified')
									 ->setSubject("Patients Appointment Report Excel")
									 ->setTitle('Patients Appointment Report')
									 ->setDescription("List of Patients")
									 ->setCategory("Reports Generation");

		//set cell A1 content with some text
		$this->excel->getActiveSheet()->setCellValue('A1', 'Patients Appointment Report');
		//change the font size
		$this->excel->getActiveSheet()->getStyle('A1')->getFont()->setSize(16);
		//merge cell A1 until D1
		$this->excel->getActiveSheet()->mergeCells('A1:I1');
		//set aligment to center for that merged cell (A1 to D1)

		$this->excel->getActiveSheet()->setCellValue('A2', 'Date: ' . $dateRange);
		$this->excel->getActiveSheet()->getStyle('A2')->getFont()->setSize(12);
		$this->excel->getActiveSheet()->mergeCells('A2:I2');

		// SET TABLE HEADER TO CENTER
		$this->excel->getActiveSheet()->getStyle('A4:I4')->getAlignment()->setHorizontal(PHPExcel_Style_Alignment::HORIZONTAL_CENTER);

		// AUTO SIZE TABLE HEADER
		foreach(range('A4','I4') as $columnID) {
		    $this->excel->getActiveSheet()->getColumnDimension($columnID)
		        ->setAutoSize(true);
		}

		// SET BACKGROUND COLOR TABLE HEADER
		$this->excel->getActiveSheet()->getStyle('A4:I4')->getFill()->applyFromArray(array(
		        'type' => PHPExcel_Style_Fill::FILL_SOLID,
		        'startcolor' => array(
		             'rgb' => 'EEEEEE'
		        )
		    ));

		$this->excel->setActiveSheetIndex(0)
		                            ->setCellValue('A4', 'CMID')
		                            ->setCellValue('B4', 'LAST NAME')
		                            ->setCellValue('C4', 'FIRST NAME')
		                            ->setCellValue('D4', 'ADDRESS')
		                            ->setCellValue('E4', 'GENDER')
		                            ->setCellValue('F4', 'PHONE NUMBER')
		                            ->setCellValue('G4', 'BIRTHDATE')
		                            ->setCellValue('H4', 'EMAIL ADDRESS')
		                            ->setCellValue('I4', 'DATE OF APPOINTMENT');



		// $dateRange = $this->input->post('date_range');
		$data = array('dateRange'=>(trim($dateRange) == FALSE) ? "" : $dateRange);
		$data = $this->patients_model->getPatientsAppReport();

		if($data)
		{
			$i = 5;
			foreach($data as $row){
				$appDate = date('M d, Y', strtotime($row->start_date));
				$startTime = date('H:i A', strtotime($row->start_date));
				$endTime = date('H:i A', strtotime($row->end_date));

			    $this->excel->setActiveSheetIndex(0)
			            ->setCellValue('A'.$i, $row->cmid)
			            ->setCellValue('B'.$i, $row->lastname)
			            ->setCellValue('C'.$i, $row->firstname)
			            ->setCellValue('D'.$i, $row->home_address.' '.$row->city.' '.$row->province)
			            ->setCellValue('E'.$i, $row->gender)
			            ->setCellValue('F'.$i, $row->phone_number)
			            ->setCellValue('G'.$i, $row->dob)
			            ->setCellValue('H'.$i, $row->email_address)
			            ->setCellValue('I'.$i, $appDate ." " . $startTime . " - " . $endTime);
			    $i++;
			}
		}

		

		$filename = "patients_apppointment_report_".date("YmdHis").'.xlsx';
		header('Content-Type: application/vnd.openxmlformats-officedocument.spreadsheetml.sheet'); //mime type
		header('Content-Disposition: attachment;filename="'.$filename.'"'); //tell browser what's the file name
		header('Cache-Control: max-age=0'); //no cache
		            
		//save it to Excel5 format (excel 2003 .XLS file), change this to 'Excel2007' (and adjust the filename extension, also the header mime type)
		//if you want to save it as .XLSX Excel 2007 format
		$objWriter = PHPExcel_IOFactory::createWriter($this->excel, 'Excel2007');  
		//force user to download the Excel file without writing it to server's HD
		$objWriter->save('php://output');
	}


	

	

}