<?php 

if ( ! defined('BASEPATH')) exit('No direct script access allowed');

require APPPATH.'controllers/General.php'; 

class Patients_report extends General{

	function __construct() {
		parent::__construct();	

		if(General::isLoggedIN() === FALSE OR General::validateToken() === FALSE){
            redirect(base_url());    
        }

        General::ModAccess(__CLASS__);
        self::getPermissionAccess();
        
        $this->load->model('patients_model');
	}

	private function getPermissionAccess()
	{
		$this->data['printPermission'] = General::validatePermission('print_report', FALSE);
        $this->data['excelPermission'] = General::validatePermission('export_excel', FALSE);
        $this->data['pdfPermission'] = General::validatePermission('export_PDF', FALSE);
        $this->data['csvPermission'] = General::validatePermission('export_CSV', FALSE);
	}

	public function index()
	{
		$this->data['page_title'] = 'Reports';
		$this->data['page_sub_title'] = 'Patients Report';
		$this->data['css'] = $this->assets_resources->getAssets('patients_report_asset')['index']['css'];
		$this->data['js'] = $this->assets_resources->getAssets('patients_report_asset')['index']['js'];
		$this->data['jsextend'] = $this->assets_resources->getAssets('patients_report_asset')['index']['jsextend'];
		$this->data['file'] = $this->assets_resources->getAssets('patients_report_asset')['index']['file'];
		$this->load->view("page", $this->data);
	}


	public function print_report()
	{

		$printOption = $this->input->post("printOption");

		if($printOption == "web_print")
		{
			$this->result_print();

			$log = 'Patients report has been printed by ' .General::getFullName();
			General::log('PATIENTS REPORT', 'PRINT', $log);

		}
		else if($printOption == "export_excel")
		{
			$this->result_excel();

			$log = 'Patients report has been exported to excel by ' .General::getFullName();
			General::log('PATIENTS REPORT', 'EXPORT EXCEL', $log);

		}
		else if($this->input->post('printOption') == "export_pdf")
		{
			$this->result_pdf();

			$log = 'Patients report has been exported to PDF by ' .General::getFullName();
			General::log('PATIENTS REPORT', 'EXPORT PDF', $log);

		}
		else if($this->input->post('printOption') == "export_csv")
		{
			$this->ExportCSV();

			$log = 'Patients report has been exported to CSV by ' .General::getFullName();
			General::log('PATIENTS REPORT', 'EXPORT CSV', $log);

		}

		
	}

	public function result_pdf()
	{
		$dateRange = $this->input->post('date_range');
		$data = array('dateRange'=>(trim($dateRange) == FALSE) ? "" : $dateRange);
		$this->data['page_title'] = "Patients Report";
		$this->data['print'] = FALSE;
		$this->data['rsDetails'] = $this->patients_model->getAll('', '', TRUE, FALSE);
		$html = $this->load->view('patients_report/print', $this->data, true);
		$filename = "patients_report_".date("YmdHis");
		$papersize = "A4";
		$orientation = $this->input->post('pdfOrientation');
        pdf_create($html, $filename, TRUE, $papersize, $orientation);
	}

	public function result_print()
	{
		$dateRange = $this->input->post('date_range');
		$data = array('dateRange'=>(trim($dateRange) == FALSE) ? "" : $dateRange);
		$this->data['page_title'] = "Patients Report";
		$this->data['print'] = TRUE;
		$this->data['rsDetails'] = $this->patients_model->getAll('', '', TRUE, FALSE);
		$this->load->view('patients_report/print', $this->data);
	}

	public function ExportCSV()
	{
        $this->load->dbutil();
        $this->load->helper('file');
        $this->load->helper('download');
        $delimiter = ",";
        $newline = "\r\n";
        $filename = "patients_report_".date("YmdHis").'.csv';

		$result = $this->patients_model->getAll('', '', TRUE, FALSE, TRUE);

        $data = $this->dbutil->csv_from_result($result, $delimiter, $newline);
        force_download($filename, $data);
	}


	public function result_excel()
	{
		//load our new PHPExcel library
		$this->load->library('excel');
		//activate worksheet number 1
		$this->excel->setActiveSheetIndex(0);
		//name the worksheet
		$this->excel->getActiveSheet()->setTitle('Patients Report');
		$this->excel->getProperties()->setCreator("Company Name")
									 ->setLastModifiedBy('Name of Modified')
									 ->setSubject("Patients Report Excel")
									 ->setTitle('Patients Report')
									 ->setDescription("List of Patients")
									 ->setCategory("Reports Generation");

		$this->excel->setActiveSheetIndex(0)
		                            ->setCellValue('A1', 'CMID')
		                            ->setCellValue('B1', 'LAST NAME')
		                            ->setCellValue('C1', 'FIRST NAME')
		                            ->setCellValue('D1', 'PHONE NUMBER')
		                            ->setCellValue('E1', 'DATE OD BIRTH')
		                            ->setCellValue('F1', 'age')
		                            ->setCellValue('G1', 'EMAIL ADDRESS');



		$dateRange = $this->input->post('date_range');
		$data = array('dateRange'=>(trim($dateRange) == FALSE) ? "" : $dateRange);
		$data = $this->patients_model->getAll('', '', TRUE, FALSE);

		if($data)
		{
			$i = 2;
			foreach($data as $row){
				$birthDate = explode("-", $row->dob);
				$age = (date("md", date("U", mktime(0, 0, 0, $birthDate[1], $birthDate[2], $birthDate[0]))) > date("md")
			    ? ((date("Y") - $birthDate[0]) - 1)
			    : (date("Y") - $birthDate[0]));

			    $this->excel->setActiveSheetIndex(0)
			            ->setCellValue('A'.$i, $row->cmid)
			            ->setCellValue('B'.$i, $row->lastname)
			            ->setCellValue('C'.$i, $row->firstname)
			            ->setCellValue('D'.$i, $row->phone_number)
			            ->setCellValue('E'.$i, $row->dob)
			            ->setCellValue('F'.$i, $age)
			            ->setCellValue('G'.$i, $row->email_address);
			    $i++;
			}
		}

		

		$filename = "patients_report_".date("YmdHis").'.xlsx';
		header('Content-Type: application/vnd.openxmlformats-officedocument.spreadsheetml.sheet'); //mime type
		header('Content-Disposition: attachment;filename="'.$filename.'"'); //tell browser what's the file name
		header('Cache-Control: max-age=0'); //no cache
		            
		//save it to Excel5 format (excel 2003 .XLS file), change this to 'Excel2007' (and adjust the filename extension, also the header mime type)
		//if you want to save it as .XLSX Excel 2007 format
		$objWriter = PHPExcel_IOFactory::createWriter($this->excel, 'Excel2007');  
		//force user to download the Excel file without writing it to server's HD
		$objWriter->save('php://output');
	}


	

	

}