<?php
defined('BASEPATH') OR exit('No direct script access allowed');

require APPPATH.'controllers/General.php'; 


class System_parameters extends General {

	private $editPermission;

	function __construct() {
		parent::__construct();	

		if(General::isLoggedIN() === FALSE OR General::validateToken() === FALSE){
            redirect(base_url());    
        }
        General::ModAccess(__CLASS__);
        self::getPermissionAccess();
        
        $this->load->model('sys_param_model');
	}

	private function getPermissionAccess()
	{
		$this->data['addPermission'] = General::validatePermission('add', FALSE);
        $this->data['deletePermission'] = General::validatePermission('delete', FALSE);
        $this->editPermission = General::validatePermission('edit', FALSE);
        $this->data['editPermission'] = $this->editPermission;
	}

	public function index()
	{
		$this->data['page_title'] = $this->lang->line('system_param_title');
		$this->data['page_sub_title'] = $this->lang->line('system_param_subtitle');
		$this->data['css'] = $this->assets_resources->getAssets('system_param_asset')['index']['css'];
		$this->data['js'] = $this->assets_resources->getAssets('system_param_asset')['index']['js'];
		$this->data['jsextend'] = $this->assets_resources->getAssets('system_param_asset')['index']['jsextend'];
		$this->data['file'] = $this->assets_resources->getAssets('system_param_asset')['index']['file'];
		$this->data['general_params'] = $this->sys_param_model->getAllParams(array('CURRENCY','RELIGION','GENDER','CIVIL_STATUS'));
		$this->load->view("page", $this->data);
	}

	public function listParam($param)
	{
		$this->data['paramList'] = $this->sys_param_model->getParamList($param);
		$this->data['param'] = $param;
		$this->load->view('system_parameters/listParam', $this->data);
	}

	public function getAllParam($param)
	{
		$this->data['paramList'] = $this->sys_param_model->getParamList($param);
		$this->data['param'] = $param;
		$this->load->view('system_parameters/getAllParam', $this->data);
	}

	public function saveParam()
	{
		General::addJSONResponseHeader();

		try {
			$this->form_validation->set_rules("param_value","Param Name","trim|xss_clean|strip_tags|required");
			$this->form_validation->set_rules("param_desc","Description","trim|xss_clean");
			$this->form_validation->set_error_delimiters('', '');

			if($this->form_validation->run())
			{	
				$result = $this->sys_param_model->save($this->input->post('action'));

				if($result) 
				{
					// Record activity log
					$log = 'New System Parameters: <strong>'. $this->input->post('param_value') . '</strong> has been added by ' .General::getFullName();
					General::log('SYSTEM PARAMETERS', 'INSERT', $log);

					$data_arr = array('success' => TRUE, 'message' => $this->lang->line('manage_param_successmsg'), 'formaction' => 'reset', 'action' => $this->input->post('action'));

				} else {
					$data_arr = array('success' => FALSE, 'message' => $this->lang->line('manage_param_errormsg'));
				}
			}
			else
			{
				$data_arr = array('success' => FALSE, 'message' => validation_errors());
			}
		} catch(Exception $e) {
			$data_arr = array("success" => FALSE, "message" => $e->getMessage());
		}

		die(json_encode($data_arr));
	}

	public function deleteParam($param_id)
	{
		General::addJSONResponseHeader();

		try {
			
			$result = $this->sys_param_model->deleteParam($param_id);

			if($result) {
				// Record activity log
				$log = 'New System Parameters with id: <strong>'. $param_id . '</strong> has been deleted by ' .General::getFullName();
				General::log('SYSTEM PARAMETERS', 'DELETE', $log);

				$data_arr = array('success' => TRUE, 'message' => $this->lang->line('manage_param_successdelmsg'), 'formaction' => 'reset');
			} else {
				$data_arr = array('success' => FALSE, 'message' => $this->lang->line('manage_param_errordelmsg'));
			}

		} catch(Exception $e) {
			$data_arr = array("success" => FALSE, "message" => $e->getMessage());
		}

		die(json_encode($data_arr));
	}

	public function editParam($param_id)
	{
		General::addJSONResponseHeader();
		$result = $this->sys_param_model->getParamDetails($param_id);
		die(json_encode($result));
	}

	public function addNewQuick($param_code, $fk)
    {
    	General::validatePermission('add');

		if(isset($_POST['param_value']) AND $_POST['param_value'] != '') 
		{
			self::saveParam();
		} 
		else 
		{
			$this->data['form_title'] = $this->lang->line('system_param_addnew');
			$this->data['param_code'] = $param_code;
			$this->data['fk'] = $fk;
        	$this->load->view('quick_forms/add_parameters',$this->data);
		}
    }
	
}
