<?php 
/**
 * Aspen Development Corporation
 *
 * @package	Aspen Development Corporation
 * @author	Jayson Sarino
 * @link	http://www.aspendevelopmentcorporation.com
 * @since	Version 1.0.0
 */
if ( ! defined('BASEPATH')) exit('No direct script access allowed'); 

/**
 * Designation Class
 *
 * @package		Aspen Development Corporation
 * @category	Model
 * @author		Jayson Sarino
 */
class Appointment_model extends CI_Model{
	
	public function __construct()
	{
		parent::__construct();	
	}

	public function getDetails($id)
	{
		$this->db->select("C.firstname as docFname, C.lastname as docLname, A.id, A.patient_id, A.start_date, A.end_date, A.duration, A.assgined_doctor, A.appointment_reason, A.date_visit, A.appointment_status, A.appointment_type, A.date_created,
			B.firstname, B.lastname");
		$this->db->join('patients B','B.id = A.patient_id','LEFT OUTER');
		$this->db->join('users C','C.user_id = A.assgined_doctor','LEFT OUTER');
		$this->db->where('A.id', $id);
		$query = $this->db->get('appointment A');
		return ($query->num_rows() == 1) ? $query->row() : NULL;
	}


	public function getAll($limit = 0, $offset = 0, $search = FALSE, $viewAll = FALSE, $exportCSV = FALSE)
	{
		$this->db->select('A.start_date, A.end_date, A.assgined_doctor, A.appointment_reason, A.date_created, B.cmid, B.firstname, B.lastname, B.dob, B.phone_number, B.email_address');
		if($viewAll === TRUE) {
			$this->db->where_in('A.nStatus', array(0, 1));
		} else {
			$this->db->where('A.nStatus', 1);
		}
		if($limit > 0 OR $offset > 0) {
			$this->db->limit($limit, $offset);
		}
		$this->db->join('patients B','B.id = A.patient_id','LEFT OUTER');
		$query = $this->db->get('appointment A');
		if($exportCSV === TRUE){
			return $query;
		} else {
			return ($query->num_rows() > 0) ? $query->result() : NULL;
		}	
		
	}

	

	public function save($action, $end = '')
    {
    	if($action == "insert") {
    		$this->data = array(
				'patient_id'				=>		$this->input->post('patient_id'),
				'start_date'				=>		$this->input->post('date'),
				'assgined_doctor'			=>		$this->input->post('assgined_doctor'),
				'appointment_reason'		=>		nl2br($this->input->post('appointment_reason')),
				'appointment_type'			=>		$this->input->post('appointment_type')
			);	
			$this->db->insert("appointment",$this->data);
			$result = $this->db->affected_rows();
			return ($result) ? $this->db->insert_id() : FALSE;
    	} 
    	else if($action == "update") {
    		$this->data = array(
				'start_date'				=>		$this->input->post('date'),
				'appointment_reason'		=>		nl2br($this->input->post('appointment_reason')),
				'appointment_type'			=>		$this->input->post('appointment_type')
			);	
			$this->db->where('id', $this->input->post('appointment_id'));
			$this->db->update("appointment",$this->data);
			$result = $this->db->affected_rows();
			return $this->input->post('appointment_id');
    	} else {
    		return FALSE;
    	}
    	
    }

    public function deleteApp($id)
    {
    	$this->data = array('nStatus' => 0);	
		$this->db->where('id', $id);
		$this->db->update("appointment",$this->data);
		$result = $this->db->affected_rows();
		return $id;
    }

    public function doneApp($id)
    {
    	$this->data = array('appointment_status' => 2);	
		$this->db->where('id', $id);
		$this->db->update("appointment",$this->data);
		$result = $this->db->affected_rows();
		return $id;
    }

    public function loadDataCalendar()
	{
		$query = $this->db->query("
			SELECT 
				A.`id` as 'id', 
				A.`assgined_doctor` as 'resourceId',
				A.`start_date` as 'start',
				A.`end_date` as 'end',
				B.`firstname`,
				B.`lastname`,
				A.`appointment_reason`,
				A.`appointment_status`,
				A.`appointment_type`
			FROM `appointment` A LEFT OUTER JOIN `patients` B ON A.`patient_id` = B.`id` 
			LEFT OUTER JOIN `users` C ON C.`user_id` = A.`assgined_doctor`
			WHERE A.`nStatus` = 1
		");
		$result = $query->result();
		$app = [];
		foreach($result as $value){
			if($value->appointment_type == 1) {
				$backgroundColor = "#1BA39C";
				$appointment_type = "Initial Consult";
			} else if($value->appointment_type == 2) {
				$backgroundColor = "#4C87B9";
				$appointment_type = "Follow Up";
			} else if($value->appointment_type == 3) {
				$backgroundColor = "#E7505A";
				$appointment_type = "Proposed";
			}
			
			$title = "Patient: ".$value->firstname.' '.$value->lastname.' \n sad';
			$title = htmlspecialchars($title);
			$app[] = array(
						'id' 					=> 	$value->id,
						'resourceId' 			=> 	$value->resourceId,
						'start' 				=> 	$value->start,
						'end' 					=> 	$value->end,
						'patient' 				=> 	$value->firstname.' '.$value->lastname,
						'appointment_reason'	=> 	$value->appointment_reason,
						'appointment_type'		=>  $appointment_type,
						'backgroundColor' 		=> 	$backgroundColor,
						'borderColor'			=>	$backgroundColor
					);
		}
		return $app;
	}

	public function getAllPhysicians()
	{
		$query = $this->db->query(" SELECT `user_id` as 'id',CONCAT(firstname,' ',lastname) as 'title' FROM `users` WHERE `nStatus` = 1 AND `group_id` = 5");
		return $query->result();
	}

	public function getAllPhysiciansConsult()
	{
		$query = $this->db->query(" SELECT * FROM `users` WHERE `nStatus` = 1 AND `group_id` = 5");
		return $query->result();
	}

	public function myAppointments($today = FALSE, $all = FALSE, $forTomorrow = FALSE)
	{
		$this->db->select('A.appointment_status, A.id as appID, A.start_date, A.end_date, A.appointment_type, A.duration, A.assgined_doctor, A.appointment_reason, A.date_created, B.cmid, B.firstname, B.lastname, B.dob, B.phone_number, B.email_address, B.id');
		$this->db->where(array(
			'A.nStatus'		=>	1
		));
		if($today === TRUE){
			$this->db->where("DATE_FORMAT(A.start_date,'%Y-%m-%d')", date('Y-m-d'));
		} 

		if($all === TRUE) {
			$this->db->where("DATE_FORMAT(A.start_date,'%Y-%m-%d') >= ", date('Y-m-d'));
		}

		if($forTomorrow === TRUE) {
			$this->db->where("DATE_FORMAT(A.start_date,'%Y-%m-%d')", date('Y-m-d', strtotime("+1 day")) );
		}

		$this->db->join('patients B','B.id = A.patient_id','LEFT OUTER');
		$this->db->order_by('A.id','ASC');
		$query = $this->db->get('appointment A');
		return ($query->num_rows() > 0) ? $query->result() : NULL;	
	}

	
	
}