<?php 
/**
 * Aspen Development Corporation
 *
 * @package	Aspen Development Corporation
 * @author	Jayson Sarino
 * @link	http://www.aspendevelopmentcorporation.com
 * @since	Version 1.0.0
 */
if ( ! defined('BASEPATH')) exit('No direct script access allowed'); 

/**
 * General Class
 *
 * @package		Aspen Development Corporation
 * @category	Model
 * @author		Jayson Sarino
 */
class General_model extends CI_Model{
	
	public function __construct()
	{
		parent::__construct();	
		$this->load->model('user_model');
	} 

    public function getParamList($param)
    {
        $this->db->where(array(
            'param_code'    =>  strtoupper($param),
            'nStatus'       =>  1
        ));
        $query = $this->db->get('system_parameters');
        return ($query->num_rows() > 0) ? $query->result() : NULL;
    }

	public function getSystemSettings($settings_code)
	{
		$this->db->where('settings_code', $settings_code);
		$query = $this->db->get('system_settings');
		return $query->row();
	}

	public function log($module, $event, $log)
	{
		$this->data = array(
			'user_id'				=>		$this->session->userdata('user_id'),
			'module'				=>		$module,
			'event'					=>		$event,
			'ipaddress'				=>		$this->input->ip_address(),
			'log_description'		=>		$log,
            'date'                  =>      date('Y-m-d h:i:s A'),
		);	
		$this->db->insert("activity_log",$this->data);
	}

	public function validateToken()
	{
		$this->db->select('token_id, session_id');
		$this->db->where(array(
			'token_id'		=>	$this->session->userdata('token_id'),
			'session_id'	=>	$this->session->userdata('session_id'),
		));
		$query = $this->db->get('users');
		return ($query->num_rows() == 1) ? TRUE : FALSE;
	}

	public function getParentMenus($privileges = '', $where_in = '')
    {
    	$this->db->select('menu_id, menu_name, menu_link, menu_icon, parent_id');
    	$this->db->where('nStatus', 1);
    	if ($privileges != '') {
    		$this->db->where('parent_id', 0);
    		$this->db->where_in('menu_id', $privileges );
    	}
    	if ($where_in != '') {
    		$this->db->where_in('menu_id', $where_in );
    		$this->db->group_by('parent_id');
    	}
    	$this->db->order_by('menu_sort','ASC');
    	$query = $this->db->get('menu');
    	return ($query->num_rows() > 0 ) ? $query->result() : FALSE;
    }

    public function getMainMenus()
    {
        $this->db->select('menu_id, menu_name, menu_link, menu_icon, parent_id');
        $this->db->where('nStatus', 1);
         $this->db->where('parent_id', 0);
        $this->db->order_by('menu_sort','ASC');
        $query = $this->db->get('menu');
        return ($query->num_rows() > 0 ) ? $query->result() : FALSE;
    }

    public function getChildMenus($parent_id, $where_in = TRUE)
    {
    	$rsUserDetails = $this->user_model->getUserDetails($this->session->userdata('username'));
    	$privileges = unserialize($rsUserDetails->privileges);

    	$this->db->select('menu_id, menu_name, menu_link, menu_icon');
    	$this->db->where(array(
    			'parent_id'	=>	$parent_id,
    			'nStatus'	=>	1
    		));
    	if ($where_in === TRUE) {
    		$this->db->where_in('menu_id', $privileges );
    	}
    	$this->db->order_by('menu_sort','ASC');
    	$query = $this->db->get('menu');
    	return ($query->num_rows() > 0 ) ? $query->result() : FALSE;
    }

    public function getMenuActions($menu_id)
    {
    	$this->db->select('permission_id, permission_name, group_id, menu_id');
    	$this->db->where('menu_id', $menu_id);
    	$this->db->order_by('permission_id','ASC');
    	$query = $this->db->get('group_permission');
    	return ($query->num_rows() > 0 ) ? $query->result() : FALSE;
    }

    public function getSystemOptions($cCode, $where_not)
    {
        $this->db->where('cCode', $cCode);
        $this->db->order_by('cValue','ASC');
        if($where_not !== NULL) {
            $this->db->where_not_in('cValue', $where_not);
        }
        $query = $this->db->get('system_option');
        return ($query->num_rows() > 0 ) ? $query->result() : FALSE;
    }

    public function setLang($lang)
    {
        $this->data = array(
            'settings_value'  =>  $lang
        );  
        $this->db->where('settings_code', 'lang');
        $result = $this->db->update("system_settings",$this->data);
        return $result;
    }


    public function delete($table, $where_field, $id)
    {
        $this->db->query("DELETE FROM ".$table." WHERE ".$where_field." = " . $id);
        return $this->db->affected_rows();
    }

    public function deactivate($table, $where_field, $id)
    {
        $this->db->query("UPDATE ".$table." SET `nStatus` = 0 WHERE ".$where_field." = " . $id);
        return $this->db->affected_rows();
    }

    public function activate($table, $where_field, $id)
    {
        $this->db->query("UPDATE ".$table." SET `nStatus` = 1 WHERE ".$where_field." = " . $id);
        return $this->db->affected_rows();
    }

    public function done($table, $where_field, $id)
    {
        $this->db->query("UPDATE ".$table." SET `nStatus` = 2 WHERE ".$where_field." = " . $id);
        return $this->db->affected_rows();
    }

    public function getDeleteName($table, $whereField, $fieldName = '', $id = '')
    {
        if($fieldName != '') {
            $this->db->select($fieldName);
        } 
        $this->db->where($whereField, $id);
        $query = $this->db->get($table);
        return $query->row();
    }

    public function getUserTransactionLog($table, $field_id, $field_id_value)
    {
        $this->db->select("A.createdDateTime, A.modifiedDateTime, CONCAT(B.firstname,' ',B.lastname) as CreatedBy, 
            CONCAT(C.firstname,' ',C.lastname) as ModifiedBy");
        $this->db->where('A.' .$field_id, $field_id_value);
        $this->db->join('users B','B.user_id = A.createdByUserID','LEFT OUTER');
        $this->db->join('users C','C.user_id = A.modifiedByUserID','LEFT OUTER');
        $query = $this->db->get($table . ' as A');
        return ($query->num_rows() == 1) ? $query->row() : NULL;
    }


    function update_record($table="",$data="",$where="")
    { 
        $this->db->where($where);
        $this->db->update($table, $data);

        if ($this->db->affected_rows() > 0) {
            return true;
        } else {
            return false;
        }
    }


    function get_all_records($table="",$where="",$order="",$limit="")
    {   
        if($order)
        {
            $this->db->order_by($order,'DESC');
        }

        if($limit)
        {
            $this->db->limit($limit);
        }

        if ($where) {
            $query=$this->db->get_where($table, $where);
        } else {
            $query=$this->db->get($table);
        }

        if ($query->num_rows() > 0) {
            return $query->result();
        } else {
            return false;
        }
    }


    function getsingle($table, $where = '', $fld = NULL, $order_by = '', $order = '') {

        if ($fld != NULL) {

            $this->db->select($fld);

        }

        $this->db->limit(1);

        if ($order_by != '') {

            $this->db->order_by($order_by, $order);

        }

        if ($where != '') {

            $this->db->where($where);

        }

        $q = $this->db->get($table);

        return $q->row();

    }


    function getAll($table, $where = '',$select = 'all', $order_fld = '', $order_type = '', $limit = '', $offset = '',$group_by='') 
    {

        $data = array();

        if ($select == 'all') {

            $this->db->select('*');

        } else {

            $this->db->select($select);

        }

        if ($where != '') {

            $this->db->where($where);

        }

        if($group_by !=''){

            $this->db->group_by($group_by);

        }

        $this->db->from($table);

        if ($limit != '' && $offset != '') {

            $this->db->limit($limit, $offset);

        } else if ($limit != '') {

            $this->db->limit($limit);

        }

        if ($order_fld != '' && $order_type != '') {

            $this->db->order_by($order_fld, $order_type);

        }

        $q = $this->db->get();

        return $q->result();

    }


    function insertData($table, $dataInsert) {

        $this->db->insert($table, $dataInsert);

        return $this->db->insert_id();

    }


    function updateFields($table, $data, $where){

        $this->db->where($where);

        $update = $this->db->update($table, $data);

        if($update){

            return true;

        }else{

            return false;

        }

    }
	
}