<?php 
/**
 * Aspen Development Corporation
 *
 * @package	Aspen Development Corporation
 * @author	Jayson Sarino
 * @link	http://www.aspendevelopmentcorporation.com
 * @since	Version 1.0.0
 */
if ( ! defined('BASEPATH')) exit('No direct script access allowed'); 

/**
 * Designation Class
 *
 * @package		Aspen Development Corporation
 * @category	Model
 * @author		Jayson Sarino
 */
class Patients_model extends CI_Model{
	
	public function __construct()
	{
		parent::__construct();	
	}

	public function getDetails($id)
	{
		$this->db->where('id', $id);
		$query = $this->db->get('patients');
		return ($query->num_rows() == 1) ? $query->row() : NULL;
	}


	public function getAll($limit = 0, $offset = 0, $search = FALSE, $viewAll = FALSE, $exportCSV = FALSE, $sort = FALSE)
	{
		$this->db->select('*');
		if($search === TRUE) {
			if(!empty($_POST['cmid'])) {
				$this->db->or_like('cmid', $_POST['cmid']);
			}
			if(!empty($_POST['patient_name'])) {
				$this->db->or_like('lastname', $_POST['patient_name']);
				$this->db->or_like('firstname', $_POST['patient_name']);
			}
			if(!empty($_POST['gender'])) {
				$this->db->or_like('gender', $_POST['gender']);
			}
			if(!empty($_POST['email_address'])) {
				$this->db->or_like('email_address', $_POST['email_address']);
			}
			if(!empty($_POST['phone_number'])) {
				$this->db->or_like('phone_number', $_POST['phone_number']);
			}
			if(!empty($_POST['home_address'])) {
				$this->db->or_like('home_address', $_POST['home_address']);
				$this->db->or_like('city', $_POST['home_address']);
				$this->db->or_like('province', $_POST['home_address']);
				$this->db->or_like('postal_code', $_POST['home_address']);
			}
			if(!empty($_POST['date_range']) AND isset($_POST['date_range'])){
				$dateRange = $this->input->post('date_range');
				$dateStart = date("Y-m-d", strtotime(explode("-", $dateRange)[0]));
				$dateend = date("Y-m-d", strtotime(explode("-", $dateRange)[1]));
				$this->db->where("DATE_FORMAT(date_created, '%Y-%m-%d') >= '".$dateStart."' AND DATE_FORMAT(date_created, '%Y-%m-%d') <= '".$dateend."'");
			}

			if(isset($_POST['nStatus']) AND !empty($_POST['nStatus'])){
				if($_POST['nStatus'] == 1) {
					$this->db->or_where('nStatus', $_POST['nStatus']);
				} else if($_POST['nStatus'] == 0) {
					$this->db->or_where('nStatus', $_POST['nStatus']);
				} 
			}
			
		}
		if($viewAll === TRUE) {
			$this->db->where_in('nStatus', array(0, 1));
		} else {
			$this->db->where('nStatus', 1);
		}
		if($limit > 0 OR $offset > 0) {
			$this->db->limit($limit, $offset);
		}
		if($sort === TRUE){
			$this->db->order_by('', '');
		}
		$query = $this->db->get('patients');
		if($exportCSV === TRUE){
			return $query;
		} else {
			return ($query->num_rows() > 0) ? $query->result() : NULL;
		}	
		
	}

	public function getLastGestational($id)
	{
		$query = $this->db->query("SELECT A.*,B.firstname as cFName, B.lastname as cLName, C.firstname as mFName, C.lastname as mLName
								FROM `patient_gestational_age` A LEFT OUTER JOIN `users` B ON B.`user_id` = A.`createdByUserID`
								LEFT OUTER JOIN `users` C ON C.`user_id` = A.`modifiedByUserID`
								WHERE A.`patient_id` = ".$id." AND A.`nStatus` = 1 
								ORDER BY A.`age_id` DESC LIMIT 1");
		if($query->num_rows() == 1) {
			return $query->row();
		} else {
			return FALSE;
		}
	}

	public function saveNotes($action, $patient_id = '')
	{
		if($action == "insert") {
			$this->data = array(
				'notes'					=>		nl2br($this->input->post('notes')),
				'patient_id'			=>		$patient_id,
				'createdByUserID'		=>		$this->session->userdata('user_id')
			);	
			$this->db->insert("patient_notes",$this->data);
			$result = $this->db->affected_rows();
			return ($result) ? $this->db->insert_id() : FALSE;
		}
	}

	public function saveGestationalAge($action, $patient_id = '')
	{
		if($action == "insert") {
			$this->data = array(
				'estimated_delivery_date'		=>		$this->input->post('estimated_delivery_date'),
				'gestation_week_day'			=>		$this->input->post('gestation_week_day'),
				'patient_id'					=>		$patient_id,
				'createdByUserID'				=>		$this->session->userdata('user_id')
			);	
			$this->db->insert("patient_gestational_age",$this->data);
			$result = $this->db->affected_rows();
			return ($result) ? $this->db->insert_id() : FALSE;
		}
	}

	public function savePatientRecord($action, $patient_id = '')
	{
		if($action == "insert") {
			$this->data = array(
				'weight'				=>		$this->input->post('weight'),
				'bp'					=>		$this->input->post('bp'),
				'temp'					=>		$this->input->post('temp'),
				'age_gestation'			=>		$this->input->post('age_gestation'),
				'chief_complain'		=>		nl2br($this->input->post('chief_complain')),
				'fetal_heart_beat'		=>		nl2br($this->input->post('fetal_heart_beat')),
				'internal_exam'			=>		nl2br($this->input->post('internal_exam')),
				'lab_results'			=>		nl2br($this->input->post('lab_results')),
				'comments'				=>		nl2br($this->input->post('comments')),
				'patient_id'			=>		$patient_id,
				'date_created'			=>		date('Y-m-d h:i:s A'),
				'createdByUserID'		=>		$this->session->userdata('user_id')
			);	
			$this->db->insert("medical_docs",$this->data);
			$result = $this->db->affected_rows();
			return ($result) ? $this->db->insert_id() : FALSE;
		}
	}

	public function save($action, $cmid = '')
    {
    	if($action == "insert") {
    		$this->data = array(
				'cmid'						=>		$cmid,
				'lastname'					=>		$this->input->post('lastname'),
				'middlename'				=>		$this->input->post('middlename'),
				'firstname'					=>		$this->input->post('firstname'),
				'dob'						=>		$this->input->post('dob'),
				'gender'					=>		$this->input->post('gender'),
				'home_address'				=>		$this->input->post('home_address'),
				'email_address'				=>		$this->input->post('email_address'),
				'phone_number'				=>		$this->input->post('phone_number'),
				'health_card_number'		=>		$this->input->post('health_card_number'),
				'estimated_delivery_date'	=>		(!empty($this->input->post('estimated_delivery_date'))) ? $this->input->post('estimated_delivery_date') : NULL,
				'gestation_week_day'		=>		$this->input->post('gestation_week_day'),
				'date_created'				=>		date('Y-m-d h:i:s A'),
				'notes'						=>		$this->input->post('notes'),
			);	
			$this->db->insert("patients",$this->data);
			$result = $this->db->affected_rows();
			return ($result) ? $this->db->insert_id() : FALSE;
    	} 
    	else if($action == "update") {
    		$this->data = array(
				'cmid'						=>		$cmid,
				'lastname'					=>		$this->input->post('lastname'),
				'middlename'				=>		$this->input->post('middlename'),
				'firstname'					=>		$this->input->post('firstname'),
				'dob'						=>		$this->input->post('dob'),
				'gender'					=>		$this->input->post('gender'),
				'home_address'				=>		$this->input->post('home_address'),
				'email_address'				=>		$this->input->post('email_address'),
				'phone_number'				=>		$this->input->post('phone_number'),
				'health_card_number'		=>		$this->input->post('health_card_number'),
				'estimated_delivery_date'	=>		(!empty($this->input->post('estimated_delivery_date'))) ? $this->input->post('estimated_delivery_date') : NULL,
				'gestation_week_day'		=>		$this->input->post('gestation_week_day'),
				'notes'						=>		$this->input->post('notes'),
			);	
			$this->db->where('id', $this->input->post('id'));
			$this->db->update("patients",$this->data);
			$result = $this->db->affected_rows();
			return $result;
    	} else {
    		return FALSE;
    	}
    	
    }

    public function updateCMID($cmid)
    {
    	$this->data = array('settings_value'	=>	$cmid);	
		$this->db->where('settings_code', 'CMID');
		$this->db->update("system_settings",$this->data);
		$result = $this->db->affected_rows();
		return $result;
    }

    public function updateCMIDImport($cmid)
    {
    	$this->data = array('settings_value'	=>	$cmid);	
		$this->db->where('settings_code', 'CMID');
		$this->db->update("system_settings",$this->data);
		$result = $this->db->affected_rows();
		return $result;
    }

    public function loadMedicalDocs($patient_id)
    {
    	$this->db->select("
    		A.docs_id, A.patient_id, A.weight, A.bp, A.temp, A.age_gestation, A.chief_complain, A.fetal_heart_beat, A.internal_exam, A.lab_results, A.comments, A.file_docs,
    		A.date_created, A.date_modified, A.nStatus, A.modifiedByUserID,
    		CONCAT(B.firstname,' ',B.lastname) as createdBy, CONCAT(C.firstname,' ',C.lastname) as modifiedBy");
    	$this->db->where(array('A.patient_id' => $patient_id, 'A.nStatus' => 1));
    	$this->db->join('users B','B.user_id = A.createdByUserID','LEFT OUTER');
    	$this->db->join('users C','C.user_id = A.modifiedByUserID','LEFT OUTER');
    	$this->db->order_by('A.docs_id', 'DESC');
		$query = $this->db->get('medical_docs A');
		return ($query->num_rows() > 0) ? $query->result() : NULL;
    }

    public function delete_record($id)
    {
    	$this->data = array('nStatus'=>0);	
		$this->db->where('docs_id', $id);
		$this->db->update("medical_docs",$this->data);
		$result = $this->db->affected_rows();
		return $result;	
    }

    public function getRecord($id)
    {
    	$this->db->select("A.id, A.cmid, A.quantity, A.period_use, A.diagnosis_comments, A.thc, A.cbd, A.restrictions, B.firstname, B.lastname, 
    		B.middlename, B.dob, C.firstname as dcfname, C.lastname as dclname, C.hc_business_address, C.full_business_address, C.phone, 
    		C.province_authorize, C.hc_license_number, C.fax_number, C.email, C.signature, A.date_created");
    	$this->db->where('A.id', $id);
    	$this->db->join('patients B','B.cmid = A.cmid','LEFT OUTER');
    	$this->db->join('users C','C.user_id = A.createdByUserID','LEFT OUTER');
		$query = $this->db->get('patient_medical A');
		return ($query->num_rows() == 1) ? $query->row() : NULL;
    }

    public function getMedicalRecord($docs_id)
    {
    	$this->db->select("
    		A.docs_id, A.patient_id, A.weight, A.bp, A.temp, A.age_gestation, A.chief_complain, A.fetal_heart_beat, A.internal_exam, A.lab_results, A.comments, A.file_docs,
    		A.date_created, A.date_modified, A.nStatus, A.modifiedByUserID,
    		CONCAT(B.firstname,' ',B.lastname) as createdBy, CONCAT(C.firstname,' ',C.lastname) as modifiedBy");
    	$this->db->where(array('A.docs_id' => $docs_id, 'A.nStatus' => 1));
    	$this->db->join('users B','B.user_id = A.createdByUserID','LEFT OUTER');
    	$this->db->join('users C','C.user_id = A.modifiedByUserID','LEFT OUTER');
    	$this->db->order_by('A.docs_id', 'DESC');
		$query = $this->db->get('medical_docs A');
		return ($query->num_rows() == 1) ? $query->row() : NULL;
    }

    public function save_record($action, $upload_data = array())
    {

    	if($action == "insert") {
    		$this->data = array(
				'patient_id'			=>		$this->input->post('patient_id'),
				'weight'				=>		$this->input->post('weight'),
				'bp'					=>		$this->input->post('bp'),
				'temp'					=>		$this->input->post('temp'),
				'age_gestation'			=>		$this->input->post('age_gestation'),
				'fetal_heart_beat'		=>		nl2br($this->input->post('fetal_heart_beat')),
				'chief_complain'		=>		nl2br($this->input->post('chief_complain')),
				'internal_exam'			=>		nl2br($this->input->post('internal_exam')),
				'lab_results'			=>		nl2br($this->input->post('lab_results')),
				'comments'				=>		nl2br($this->input->post('comments')),
				'createdByUserID'		=>		$this->session->userdata('user_id'),
				'date_created'			=>		date('Y-m-d h:i:s A')
			);	
			$this->db->insert("medical_docs",$this->data);
			$result = $this->db->affected_rows();
			return ($result) ? $this->db->insert_id() : FALSE;
    	} 
    	else if($action == "update") {
    		$this->data = array(
				'weight'				=>		$this->input->post('weight'),
				'bp'					=>		$this->input->post('bp'),
				'temp'					=>		$this->input->post('temp'),
				'age_gestation'			=>		$this->input->post('age_gestation'),
				'fetal_heart_beat'		=>		nl2br($this->input->post('fetal_heart_beat')),
				'chief_complain'		=>		nl2br($this->input->post('chief_complain')),
				'internal_exam'			=>		nl2br($this->input->post('internal_exam')),
				'lab_results'			=>		nl2br($this->input->post('lab_results')),
				'comments'				=>		nl2br($this->input->post('comments')),
				'modifiedByUserID'		=>		$this->session->userdata('user_id'),
				'date_modified'			=>		date('Y-m-d h:i:s A')
			);	
			$this->db->where('docs_id', $this->input->post('docs_id'));
			$this->db->update("medical_docs",$this->data);
			$result = $this->db->affected_rows();
			return $result;
    	} else {
    		return FALSE;
    	}
    }

    public function getPatientAppointments($dateStart, $dateend, $patient_id)
    {
    	$this->db->select("A.id, A.patient_id, A.start_date, A.end_date, A.duration, B.firstname, B.lastname, A.appointment_reason, A.date_visit, 
    		A.appointment_status, A.appointment_type, A.date_created");
    	$this->db->where(array(
    			'A.nStatus'			=>		1,
    			'A.patient_id'		=>		$patient_id
    		));
    	// $this->db->where("DATE_FORMAT(A.start_date, '%Y-%m-%d') BETWEEN '".$dateStart."' AND '".$dateend."'");
    	$this->db->join('users B','B.user_id = A.assgined_doctor','LEFT OUTER');
    	$this->db->order_by('A.id','DESC');
		$query = $this->db->get('appointment A');
		return ($query->num_rows() > 0) ? $query->result() : FALSE;
    }


    public function getPatientsAppReport($exportCSV = FALSE)
	{
		$this->db->select('B.cmid, B.lastname, B.firstname, B.home_address, B.city, B.province, B.phone_number, B.gender, B.dob, B.email_address, A.start_date, A.end_date');

		if(!empty($_POST['date_range']) AND isset($_POST['date_range'])){
			$dateRange = $this->input->post('date_range');
			$dateStart = date("Y-m-d", strtotime(explode("-", $dateRange)[0]));
			$dateend = date("Y-m-d", strtotime(explode("-", $dateRange)[1]));
			$this->db->where("DATE_FORMAT(A.`start_date`, '%Y-%m-%d') >= '".$dateStart."' AND DATE_FORMAT(A.`start_date`, '%Y-%m-%d') <= '".$dateend."'");
		}
		
		$this->db->where('A.nStatus', 1);

		$this->db->order_by('A.start_date', 'ASC');
		$this->db->join('patients B','B.id = A.patient_id','LEFT OUTER');
		$query = $this->db->get('appointment A');

		if($exportCSV === TRUE){
			return $query;
		} else {
			return ($query->num_rows() > 0) ? $query->result() : NULL;
		}	
		
	}

	
	
}