<?php 
/**
 * Aspen Development Corporation
 *
 * @package	Aspen Development Corporation
 * @author	Jayson Sarino
 * @link	http://www.aspendevelopmentcorporation.com
 * @since	Version 1.0.0
 */
if ( ! defined('BASEPATH')) exit('No direct script access allowed'); 

/**
 * User Class
 *
 * @package		Aspen Development Corporation
 * @category	Model
 * @author		Jayson Sarino
 */
class User_model extends CI_Model{
	
	public function __construct()
	{
		parent::__construct();	
	}

	public function getUserDetails($username, $user_id = '')
	{
		$this->db->select('A.group_id, A.department_id, A.designation_id, A.user_id, A.firstname, A.lastname, A.email, A.phone, A.profile_picture, A.hc_city, A.hc_postalcode,
			A.token_id, A.session_id, A.fpTokenID, A.theme, A.username, A.password, D.designation_name, C.department_name, A.nStatus,
			B.privileges, B.group_name, A.hc_business_address, A.full_business_address, A.province_authorize, A.hc_license_number, A.fax_number, A.signature');
		if($user_id != '') {
			$this->db->where('A.user_id', $user_id);	
		} else {
			$this->db->where('A.username', $username);	
		}
		
		$this->db->join('user_group B', 'B.group_id = A.group_id', 'LEFT OUTER');
		$this->db->join('department C', 'C.department_id = A.department_id', 'LEFT OUTER');
		$this->db->join('designation D', 'D.designation_id = A.designation_id', 'LEFT OUTER');
		$query = $this->db->get("users A");
		return $query->row();
	}

	public function getUserLog($id)
	{
		$this->db->where('user_id', $id);	
		$this->db->order_by('log_id', 'ASC');
		$query = $this->db->get("activity_log");
		return $query->result();
	}

	public function getUserByGroup($group_id)
	{
		$this->db->select('user_id, group_id, firstname, lastname, email');
		$this->db->where('group_id', $group_id);
		$query = $this->db->get('users');
		return ($query->num_rows() > 0) ? $query->result() : NULL;
	}

	public function getUsers($limit = 0, $offset = 0, $search = FALSE, $viewAll = FALSE, $exportCSV = FALSE)
	{
		$this->db->select('A.user_id, CONCAT(A.firstname," ",A.lastname) as name, A.email, A.phone, A.nStatus, B.group_name');
		if($search === TRUE) {
			if(!empty($_POST['email'])) {
				$this->db->or_like('A.email', $_POST['email']);
			}
			if(!empty($_POST['phone'])) {
				$this->db->or_like('A.phone', $_POST['phone']);
			}
			if(!empty($_POST['fullname'])) {
				$this->db->or_like('A.firstname', $_POST['fullname']);
				$this->db->or_like('A.lastname', $_POST['fullname']);
			}
			if(!empty($_POST['group_id'])) {
				$this->db->or_where('A.group_id', $_POST['group_id']);
			}
			if($_POST['nStatus'] == 1) {
				$this->db->or_where('A.nStatus', $_POST['nStatus']);
			} else if($_POST['nStatus'] == 0) {
				$this->db->or_where('A.nStatus', $_POST['nStatus']);
			} 
		}
		if($viewAll === TRUE) {
			$this->db->where_in('A.nStatus', array(0, 1));
		} else {
			$this->db->where('A.nStatus', 1);
		}
		$this->db->join('user_group B', 'B.group_id = A.group_id', 'LEFT OUTER');
		if($limit > 0 OR $offset > 0) {
			$this->db->limit($limit, $offset);
		}
		$query = $this->db->get('users A');
		if($exportCSV === TRUE){
			return $query;
		} else {
			return ($query->num_rows() > 0) ? $query->result() : NULL;
		}	
	}

	public function inactivate($id)
	{
		$this->db->query('UPDATE `users` SET `nStatus` = 0 WHERE `user_id` = ' . $id);
	}

	public function save($action, $username = '', $password = '')
    {
    	if($action == "insert") {
    		$this->data = array(
				'firstname'			=>		$this->input->post('firstname'),
				'lastname'			=>		$this->input->post('lastname'),
				'group_id'			=>		$this->input->post('group_id'),
				'department_id'		=>		$this->input->post('department_id'),
				'designation_id'	=>		$this->input->post('designation_id'),
				'email'				=>		$this->input->post('email'),
				'phone'				=>		$this->input->post('phone'),
				'createdByUserID'	=>		$this->session->userdata('user_id'),
				'username'			=>		$username,
				'password'			=>		$password
			);	
			$this->db->insert("users",$this->data);
			$result = $this->db->affected_rows();
			return ($result) ? $this->db->insert_id() : FALSE;
    	} 
    	else if($action == "update") {
    		$this->data = array(
				'firstname'			=>		$this->input->post('firstname'),
				'lastname'			=>		$this->input->post('lastname'),
				'group_id'			=>		$this->input->post('group_id'),
				'department_id'		=>		$this->input->post('department_id'),
				'designation_id'	=>		$this->input->post('designation_id'),
				'email'				=>		$this->input->post('email'),
				'modifiedByUserID'	=>		$this->session->userdata('user_id'),
				'phone'				=>		$this->input->post('phone')
			);	
			$this->db->where('user_id', $this->input->post('user_id'));
			$this->db->update("users",$this->data);
			$result = $this->db->affected_rows();
			return $result;
    	} else {
    		return FALSE;
    	}
    	
    }

    public function save_profile()
    {
    	$this->data = array(
			'firstname'					=>		$this->input->post('firstname'),
			'lastname'					=>		$this->input->post('lastname'),
			'email'						=>		$this->input->post('email'),
			'username'					=>		$this->input->post('username'),
			'phone'						=>		$this->input->post('phone')
		);	
		$this->db->where('user_id', $this->session->userdata('user_id'));
		$this->db->update("users",$this->data);
		$result = $this->db->affected_rows();
		return $result;
    }

    public function upload_picture($upload_data = array())
    {
    	$this->data = array(
			'profile_picture'	=>		$upload_data['file_name']
		);	
		$this->db->where('user_id', $this->session->userdata('user_id'));
		$this->db->update("users",$this->data);
		$result = $this->db->affected_rows();
		return $result;
    }

    public function change_password($password, $reset = FALSE)
    {
    	
		if($reset === TRUE)
		{
			$this->data = array(
				'password'	=>		$password,
				'fpTokenID' =>		''
			);	
			$this->db->where('user_id', $this->input->post('user_id'));
		} else {
			$this->data = array(
				'password'	=>		$password
			);	
			$this->db->where('user_id', $this->session->userdata('user_id'));
		}
		
		$this->db->update("users",$this->data);
		$result = $this->db->affected_rows();
		return $result;
    }
	
	
}